/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeoroutemodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativemaps/qdeclarativegeoroutemodel_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeoroutemodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25QDeclarativeGeoRouteModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoRouteModel::qt_create_metaobjectdata<qt_meta_tag_ZN25QDeclarativeGeoRouteModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoRouteModel",
        "QML.Element",
        "RouteModel",
        "QML.AddedInVersion",
        "1280",
        "countChanged",
        "",
        "pluginChanged",
        "queryChanged",
        "autoUpdateChanged",
        "statusChanged",
        "errorChanged",
        "routesChanged",
        "measurementSystemChanged",
        "abortRequested",
        "update",
        "routingFinished",
        "QGeoRouteReply*",
        "reply",
        "routingError",
        "QGeoRouteReply::Error",
        "error",
        "errorString",
        "queryDetailsChanged",
        "pluginReady",
        "get",
        "QGeoRoute",
        "index",
        "reset",
        "cancel",
        "plugin",
        "QDeclarativeGeoServiceProvider*",
        "query",
        "QDeclarativeGeoRouteQuery*",
        "count",
        "autoUpdate",
        "status",
        "Status",
        "RouteError",
        "measurementSystem",
        "QLocale::MeasurementSystem",
        "Null",
        "Ready",
        "Loading",
        "Error",
        "NoError",
        "EngineNotSetError",
        "CommunicationError",
        "ParseError",
        "UnsupportedOptionError",
        "UnknownError",
        "UnknownParameterError",
        "MissingRequiredParameterError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pluginChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'queryChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoUpdateChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'routesChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'measurementSystemChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'abortRequested'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'routingFinished'
        QtMocHelpers::SlotData<void(QGeoRouteReply *)>(16, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'routingError'
        QtMocHelpers::SlotData<void(QGeoRouteReply *, QGeoRouteReply::Error, const QString &)>(19, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 20, 21 }, { QMetaType::QString, 22 },
        }}),
        // Slot 'queryDetailsChanged'
        QtMocHelpers::SlotData<void()>(23, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pluginReady'
        QtMocHelpers::SlotData<void()>(24, 6, QMC::AccessPrivate, QMetaType::Void),
        // Method 'get'
        QtMocHelpers::MethodData<QGeoRoute(int)>(25, 6, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::Int, 27 },
        }}),
        // Method 'reset'
        QtMocHelpers::MethodData<void()>(28, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'cancel'
        QtMocHelpers::MethodData<void()>(29, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'plugin'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider*>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'query'
        QtMocHelpers::PropertyData<QDeclarativeGeoRouteQuery*>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'count'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'autoUpdate'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'error'
        QtMocHelpers::PropertyData<enum RouteError>(21, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'measurementSystem'
        QtMocHelpers::PropertyData<QLocale::MeasurementSystem>(39, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(37, 37, QMC::EnumFlags{}).add({
            {   41, Status::Null },
            {   42, Status::Ready },
            {   43, Status::Loading },
            {   44, Status::Error },
        }),
        // enum 'RouteError'
        QtMocHelpers::EnumData<enum RouteError>(38, 38, QMC::EnumFlags{}).add({
            {   45, RouteError::NoError },
            {   46, RouteError::EngineNotSetError },
            {   47, RouteError::CommunicationError },
            {   48, RouteError::ParseError },
            {   49, RouteError::UnsupportedOptionError },
            {   50, RouteError::UnknownError },
            {   51, RouteError::UnknownParameterError },
            {   52, RouteError::MissingRequiredParameterError },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoRouteModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN25QDeclarativeGeoRouteModelE[] = {
    QMetaObject::SuperData::link<QLocale::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QDeclarativeGeoRouteModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QDeclarativeGeoRouteModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QDeclarativeGeoRouteModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN25QDeclarativeGeoRouteModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25QDeclarativeGeoRouteModelE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoRouteModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoRouteModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->pluginChanged(); break;
        case 2: _t->queryChanged(); break;
        case 3: _t->autoUpdateChanged(); break;
        case 4: _t->statusChanged(); break;
        case 5: _t->errorChanged(); break;
        case 6: _t->routesChanged(); break;
        case 7: _t->measurementSystemChanged(); break;
        case 8: _t->abortRequested(); break;
        case 9: _t->update(); break;
        case 10: _t->routingFinished((*reinterpret_cast<std::add_pointer_t<QGeoRouteReply*>>(_a[1]))); break;
        case 11: _t->routingError((*reinterpret_cast<std::add_pointer_t<QGeoRouteReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoRouteReply::Error>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 12: _t->queryDetailsChanged(); break;
        case 13: _t->pluginReady(); break;
        case 14: { QGeoRoute _r = _t->get((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoRoute*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->reset(); break;
        case 16: _t->cancel(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoRouteReply* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoRouteReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::countChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::pluginChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::queryChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::autoUpdateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::statusChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::errorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::routesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::measurementSystemChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteModel::*)()>(_a, &QDeclarativeGeoRouteModel::abortRequested, 8))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoRouteQuery* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoServiceProvider* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v) = _t->plugin(); break;
        case 1: *reinterpret_cast<QDeclarativeGeoRouteQuery**>(_v) = _t->query(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->autoUpdate(); break;
        case 4: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 6: *reinterpret_cast<enum RouteError*>(_v) = _t->error(); break;
        case 7: *reinterpret_cast<QLocale::MeasurementSystem*>(_v) = _t->measurementSystem(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlugin(*reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v)); break;
        case 1: _t->setQuery(*reinterpret_cast<QDeclarativeGeoRouteQuery**>(_v)); break;
        case 3: _t->setAutoUpdate(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setMeasurementSystem(*reinterpret_cast<QLocale::MeasurementSystem*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoRouteModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoRouteModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QDeclarativeGeoRouteModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QDeclarativeGeoRouteModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoRouteModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeGeoRouteModel::pluginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoRouteModel::queryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeoRouteModel::autoUpdateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeGeoRouteModel::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeGeoRouteModel::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeGeoRouteModel::routesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativeGeoRouteModel::measurementSystemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativeGeoRouteModel::abortRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
namespace {
struct qt_meta_tag_ZN25QDeclarativeGeoRouteQueryE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoRouteQuery::qt_create_metaobjectdata<qt_meta_tag_ZN25QDeclarativeGeoRouteQueryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoRouteQuery",
        "QML.Element",
        "RouteQuery",
        "QML.AddedInVersion",
        "1280",
        "numberAlternativeRoutesChanged",
        "",
        "travelModesChanged",
        "routeOptimizationsChanged",
        "waypointsChanged",
        "excludedAreasChanged",
        "featureTypesChanged",
        "maneuverDetailChanged",
        "segmentDetailChanged",
        "queryDetailsChanged",
        "departureTimeChanged",
        "excludedAreaCoordinateChanged",
        "waypointChanged",
        "addWaypoint",
        "QGeoCoordinate",
        "w",
        "removeWaypoint",
        "waypoint",
        "clearWaypoints",
        "addExcludedArea",
        "QGeoRectangle",
        "area",
        "removeExcludedArea",
        "clearExcludedAreas",
        "setFeatureWeight",
        "FeatureType",
        "featureType",
        "FeatureWeight",
        "featureWeight",
        "resetFeatureWeights",
        "doCoordinateChanged",
        "numberAlternativeRoutes",
        "travelModes",
        "TravelModes",
        "routeOptimizations",
        "RouteOptimizations",
        "segmentDetail",
        "SegmentDetail",
        "maneuverDetail",
        "ManeuverDetail",
        "waypoints",
        "QList<QGeoCoordinate>",
        "excludedAreas",
        "QList<QGeoRectangle>",
        "featureTypes",
        "QList<int>",
        "departureTime",
        "TravelMode",
        "CarTravel",
        "PedestrianTravel",
        "BicycleTravel",
        "PublicTransitTravel",
        "TruckTravel",
        "FeatureTypes",
        "NoFeature",
        "TollFeature",
        "HighwayFeature",
        "PublicTransitFeature",
        "FerryFeature",
        "TunnelFeature",
        "DirtRoadFeature",
        "ParksFeature",
        "MotorPoolLaneFeature",
        "TrafficFeature",
        "NeutralFeatureWeight",
        "PreferFeatureWeight",
        "RequireFeatureWeight",
        "AvoidFeatureWeight",
        "DisallowFeatureWeight",
        "RouteOptimization",
        "ShortestRoute",
        "FastestRoute",
        "MostEconomicRoute",
        "MostScenicRoute",
        "NoSegmentData",
        "BasicSegmentData",
        "SegmentDetails",
        "NoManeuvers",
        "BasicManeuvers",
        "ManeuverDetails"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'numberAlternativeRoutesChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'travelModesChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'routeOptimizationsChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waypointsChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'excludedAreasChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'featureTypesChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maneuverDetailChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'segmentDetailChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'queryDetailsChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'departureTimeChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'excludedAreaCoordinateChanged'
        QtMocHelpers::SlotData<void()>(16, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'waypointChanged'
        QtMocHelpers::SlotData<void()>(17, 6, QMC::AccessPrivate, QMetaType::Void),
        // Method 'addWaypoint'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Method 'removeWaypoint'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(21, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 22 },
        }}),
        // Method 'clearWaypoints'
        QtMocHelpers::MethodData<void()>(23, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'addExcludedArea'
        QtMocHelpers::MethodData<void(const QGeoRectangle &)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Method 'removeExcludedArea'
        QtMocHelpers::MethodData<void(const QGeoRectangle &)>(27, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Method 'clearExcludedAreas'
        QtMocHelpers::MethodData<void()>(28, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'setFeatureWeight'
        QtMocHelpers::MethodData<void(enum FeatureType, enum FeatureWeight)>(29, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { 0x80000000 | 32, 33 },
        }}),
        // Method 'featureWeight'
        QtMocHelpers::MethodData<int(enum FeatureType)>(33, 6, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Method 'resetFeatureWeights'
        QtMocHelpers::MethodData<void()>(34, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'doCoordinateChanged'
        QtMocHelpers::MethodData<void()>(35, 6, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'numberAlternativeRoutes'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'travelModes'
        QtMocHelpers::PropertyData<TravelModes>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'routeOptimizations'
        QtMocHelpers::PropertyData<RouteOptimizations>(39, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'segmentDetail'
        QtMocHelpers::PropertyData<enum SegmentDetail>(41, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'maneuverDetail'
        QtMocHelpers::PropertyData<enum ManeuverDetail>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'waypoints'
        QtMocHelpers::PropertyData<QList<QGeoCoordinate>>(45, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'excludedAreas'
        QtMocHelpers::PropertyData<QList<QGeoRectangle>>(47, 0x80000000 | 48, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'featureTypes'
        QtMocHelpers::PropertyData<QList<int>>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'departureTime'
        QtMocHelpers::PropertyData<QDateTime>(51, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9, 0x50d),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TravelMode'
        QtMocHelpers::EnumData<enum TravelMode>(52, 52, QMC::EnumFlags{}).add({
            {   53, TravelMode::CarTravel },
            {   54, TravelMode::PedestrianTravel },
            {   55, TravelMode::BicycleTravel },
            {   56, TravelMode::PublicTransitTravel },
            {   57, TravelMode::TruckTravel },
        }),
        // flag 'TravelModes'
        QtMocHelpers::EnumData<TravelModes>(38, 52, QMC::EnumIsFlag).add({
            {   53, TravelMode::CarTravel },
            {   54, TravelMode::PedestrianTravel },
            {   55, TravelMode::BicycleTravel },
            {   56, TravelMode::PublicTransitTravel },
            {   57, TravelMode::TruckTravel },
        }),
        // flag 'FeatureTypes'
        QtMocHelpers::EnumData<FeatureTypes>(58, 30, QMC::EnumIsFlag).add({
            {   59, FeatureType::NoFeature },
            {   60, FeatureType::TollFeature },
            {   61, FeatureType::HighwayFeature },
            {   62, FeatureType::PublicTransitFeature },
            {   63, FeatureType::FerryFeature },
            {   64, FeatureType::TunnelFeature },
            {   65, FeatureType::DirtRoadFeature },
            {   66, FeatureType::ParksFeature },
            {   67, FeatureType::MotorPoolLaneFeature },
            {   68, FeatureType::TrafficFeature },
        }),
        // enum 'FeatureWeight'
        QtMocHelpers::EnumData<enum FeatureWeight>(32, 32, QMC::EnumFlags{}).add({
            {   69, FeatureWeight::NeutralFeatureWeight },
            {   70, FeatureWeight::PreferFeatureWeight },
            {   71, FeatureWeight::RequireFeatureWeight },
            {   72, FeatureWeight::AvoidFeatureWeight },
            {   73, FeatureWeight::DisallowFeatureWeight },
        }),
        // enum 'RouteOptimization'
        QtMocHelpers::EnumData<enum RouteOptimization>(74, 74, QMC::EnumFlags{}).add({
            {   75, RouteOptimization::ShortestRoute },
            {   76, RouteOptimization::FastestRoute },
            {   77, RouteOptimization::MostEconomicRoute },
            {   78, RouteOptimization::MostScenicRoute },
        }),
        // flag 'RouteOptimizations'
        QtMocHelpers::EnumData<RouteOptimizations>(40, 74, QMC::EnumIsFlag).add({
            {   75, RouteOptimization::ShortestRoute },
            {   76, RouteOptimization::FastestRoute },
            {   77, RouteOptimization::MostEconomicRoute },
            {   78, RouteOptimization::MostScenicRoute },
        }),
        // enum 'SegmentDetail'
        QtMocHelpers::EnumData<enum SegmentDetail>(42, 42, QMC::EnumFlags{}).add({
            {   79, SegmentDetail::NoSegmentData },
            {   80, SegmentDetail::BasicSegmentData },
        }),
        // flag 'SegmentDetails'
        QtMocHelpers::EnumData<SegmentDetails>(81, 42, QMC::EnumIsFlag).add({
            {   79, SegmentDetail::NoSegmentData },
            {   80, SegmentDetail::BasicSegmentData },
        }),
        // enum 'ManeuverDetail'
        QtMocHelpers::EnumData<enum ManeuverDetail>(44, 44, QMC::EnumFlags{}).add({
            {   82, ManeuverDetail::NoManeuvers },
            {   83, ManeuverDetail::BasicManeuvers },
        }),
        // flag 'ManeuverDetails'
        QtMocHelpers::EnumData<ManeuverDetails>(84, 44, QMC::EnumIsFlag).add({
            {   82, ManeuverDetail::NoManeuvers },
            {   83, ManeuverDetail::BasicManeuvers },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoRouteQuery, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeoRouteQuery::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QDeclarativeGeoRouteQueryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QDeclarativeGeoRouteQueryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25QDeclarativeGeoRouteQueryE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoRouteQuery::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoRouteQuery *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->numberAlternativeRoutesChanged(); break;
        case 1: _t->travelModesChanged(); break;
        case 2: _t->routeOptimizationsChanged(); break;
        case 3: _t->waypointsChanged(); break;
        case 4: _t->excludedAreasChanged(); break;
        case 5: _t->featureTypesChanged(); break;
        case 6: _t->maneuverDetailChanged(); break;
        case 7: _t->segmentDetailChanged(); break;
        case 8: _t->queryDetailsChanged(); break;
        case 9: _t->departureTimeChanged(); break;
        case 10: _t->excludedAreaCoordinateChanged(); break;
        case 11: _t->waypointChanged(); break;
        case 12: _t->addWaypoint((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 13: _t->removeWaypoint((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 14: _t->clearWaypoints(); break;
        case 15: _t->addExcludedArea((*reinterpret_cast<std::add_pointer_t<QGeoRectangle>>(_a[1]))); break;
        case 16: _t->removeExcludedArea((*reinterpret_cast<std::add_pointer_t<QGeoRectangle>>(_a[1]))); break;
        case 17: _t->clearExcludedAreas(); break;
        case 18: _t->setFeatureWeight((*reinterpret_cast<std::add_pointer_t<enum FeatureType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum FeatureWeight>>(_a[2]))); break;
        case 19: { int _r = _t->featureWeight((*reinterpret_cast<std::add_pointer_t<enum FeatureType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->resetFeatureWeights(); break;
        case 21: _t->doCoordinateChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoRectangle >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoRectangle >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::numberAlternativeRoutesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::travelModesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::routeOptimizationsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::waypointsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::excludedAreasChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::featureTypesChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::maneuverDetailChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::segmentDetailChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::queryDetailsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoRouteQuery::*)()>(_a, &QDeclarativeGeoRouteQuery::departureTimeChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QGeoCoordinate> >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QGeoRectangle> >(); break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->numberAlternativeRoutes(); break;
        case 1: QtMocHelpers::assignFlags<TravelModes>(_v, _t->travelModes()); break;
        case 2: QtMocHelpers::assignFlags<RouteOptimizations>(_v, _t->routeOptimizations()); break;
        case 3: *reinterpret_cast<enum SegmentDetail*>(_v) = _t->segmentDetail(); break;
        case 4: *reinterpret_cast<enum ManeuverDetail*>(_v) = _t->maneuverDetail(); break;
        case 5: *reinterpret_cast<QList<QGeoCoordinate>*>(_v) = _t->waypoints(); break;
        case 6: *reinterpret_cast<QList<QGeoRectangle>*>(_v) = _t->excludedAreas(); break;
        case 7: *reinterpret_cast<QList<int>*>(_v) = _t->featureTypes(); break;
        case 8: *reinterpret_cast<QDateTime*>(_v) = _t->departureTime(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setNumberAlternativeRoutes(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setTravelModes(*reinterpret_cast<TravelModes*>(_v)); break;
        case 2: _t->setRouteOptimizations(*reinterpret_cast<RouteOptimizations*>(_v)); break;
        case 3: _t->setSegmentDetail(*reinterpret_cast<enum SegmentDetail*>(_v)); break;
        case 4: _t->setManeuverDetail(*reinterpret_cast<enum ManeuverDetail*>(_v)); break;
        case 5: _t->setWaypoints(*reinterpret_cast<QList<QGeoCoordinate>*>(_v)); break;
        case 6: _t->setExcludedAreas(*reinterpret_cast<QList<QGeoRectangle>*>(_v)); break;
        case 8: _t->setDepartureTime(*reinterpret_cast<QDateTime*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoRouteQuery::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoRouteQuery::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QDeclarativeGeoRouteQueryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeGeoRouteQuery::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoRouteQuery::numberAlternativeRoutesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeGeoRouteQuery::travelModesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoRouteQuery::routeOptimizationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeoRouteQuery::waypointsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeGeoRouteQuery::excludedAreasChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeGeoRouteQuery::featureTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeGeoRouteQuery::maneuverDetailChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativeGeoRouteQuery::segmentDetailChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativeGeoRouteQuery::queryDetailsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QDeclarativeGeoRouteQuery::departureTimeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
