/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/qdeclarativecategory_p.h>)
#  include <private/qdeclarativecategory_p.h>
#endif
#if __has_include(<private/qdeclarativecirclemapitem_p.h>)
#  include <private/qdeclarativecirclemapitem_p.h>
#endif
#if __has_include(<private/qdeclarativecontactdetails_p.h>)
#  include <private/qdeclarativecontactdetails_p.h>
#endif
#if __has_include(<private/qdeclarativegeocodemodel_p.h>)
#  include <private/qdeclarativegeocodemodel_p.h>
#endif
#if __has_include(<private/qdeclarativegeojsondata_p.h>)
#  include <private/qdeclarativegeojsondata_p.h>
#endif
#if __has_include(<private/qdeclarativegeomap_p.h>)
#  include <private/qdeclarativegeomap_p.h>
#endif
#if __has_include(<private/qdeclarativegeomapcopyrightsnotice_p.h>)
#  include <private/qdeclarativegeomapcopyrightsnotice_p.h>
#endif
#if __has_include(<private/qdeclarativegeomapitembase_p.h>)
#  include <private/qdeclarativegeomapitembase_p.h>
#endif
#if __has_include(<private/qdeclarativegeomapitemgroup_p.h>)
#  include <private/qdeclarativegeomapitemgroup_p.h>
#endif
#if __has_include(<private/qdeclarativegeomapitemview_p.h>)
#  include <private/qdeclarativegeomapitemview_p.h>
#endif
#if __has_include(<private/qdeclarativegeomapquickitem_p.h>)
#  include <private/qdeclarativegeomapquickitem_p.h>
#endif
#if __has_include(<private/qdeclarativegeoroutemodel_p.h>)
#  include <private/qdeclarativegeoroutemodel_p.h>
#endif
#if __has_include(<private/qdeclarativegeoserviceprovider_p.h>)
#  include <private/qdeclarativegeoserviceprovider_p.h>
#endif
#if __has_include(<private/qdeclarativeplace_p.h>)
#  include <private/qdeclarativeplace_p.h>
#endif
#if __has_include(<private/qdeclarativeplacecontentmodel_p.h>)
#  include <private/qdeclarativeplacecontentmodel_p.h>
#endif
#if __has_include(<private/qdeclarativepolygonmapitem_p.h>)
#  include <private/qdeclarativepolygonmapitem_p.h>
#endif
#if __has_include(<private/qdeclarativepolylinemapitem_p.h>)
#  include <private/qdeclarativepolylinemapitem_p.h>
#endif
#if __has_include(<private/qdeclarativerectanglemapitem_p.h>)
#  include <private/qdeclarativerectanglemapitem_p.h>
#endif
#if __has_include(<private/qdeclarativeroutemapitem_p.h>)
#  include <private/qdeclarativeroutemapitem_p.h>
#endif
#if __has_include(<private/qdeclarativesearchresultmodel_p.h>)
#  include <private/qdeclarativesearchresultmodel_p.h>
#endif
#if __has_include(<private/qdeclarativesearchsuggestionmodel_p.h>)
#  include <private/qdeclarativesearchsuggestionmodel_p.h>
#endif
#if __has_include(<private/qdeclarativesupportedcategoriesmodel_p.h>)
#  include <private/qdeclarativesupportedcategoriesmodel_p.h>
#endif
#if __has_include(<private/qgeomaneuverderived_p.h>)
#  include <private/qgeomaneuverderived_p.h>
#endif
#if __has_include(<private/qgeomaptype_p.h>)
#  include <private/qgeomaptype_p.h>
#endif
#if __has_include(<private/qlocationglobal_p.h>)
#  include <private/qlocationglobal_p.h>
#endif
#if __has_include(<qgeomaneuver.h>)
#  include <qgeomaneuver.h>
#endif
#if __has_include(<qgeoroute.h>)
#  include <qgeoroute.h>
#endif
#if __has_include(<qgeoroutesegment.h>)
#  include <qgeoroutesegment.h>
#endif
#if __has_include(<qplaceattribute.h>)
#  include <qplaceattribute.h>
#endif
#if __has_include(<qplacecontactdetail.h>)
#  include <qplacecontactdetail.h>
#endif
#if __has_include(<qplaceicon.h>)
#  include <qplaceicon.h>
#endif
#if __has_include(<qplaceratings.h>)
#  include <qplaceratings.h>
#endif
#if __has_include(<qplacesupplier.h>)
#  include <qplacesupplier.h>
#endif
#if __has_include(<qplaceuser.h>)
#  include <qplaceuser.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtLocation()
{
    qmlRegisterModule("QtLocation", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<QDeclarativeCategory>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativeCategory::Visibility>("QDeclarativeCategory::Visibility");
    qmlRegisterEnum<QDeclarativeCategory::Status>("QDeclarativeCategory::Status");
    qmlRegisterTypesAndRevisions<QDeclarativeCircleMapItem>("QtLocation", 6);
    qmlRegisterAnonymousTypesAndRevisions<QQuickItem>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeContactDetails>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoJsonData>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoMap>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoMapCopyrightNotice>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoMapItemBase>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoMapItemGroup>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoMapItemView>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoMapQuickItem>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeoRouteModel>("QtLocation", 6);
    qmlRegisterAnonymousTypesAndRevisions<QAbstractItemModel>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativeGeoRouteModel::Status>("QDeclarativeGeoRouteModel::Status");
    qmlRegisterEnum<QDeclarativeGeoRouteModel::RouteError>("QDeclarativeGeoRouteModel::RouteError");
    qmlRegisterTypesAndRevisions<QDeclarativeGeoRouteQuery>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::TravelMode>("QDeclarativeGeoRouteQuery::TravelMode");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::TravelModes>("QDeclarativeGeoRouteQuery::TravelModes");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::TravelMode>("QDeclarativeGeoRouteQuery::TravelMode");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::FeatureTypes>("QDeclarativeGeoRouteQuery::FeatureTypes");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::FeatureType>("QDeclarativeGeoRouteQuery::FeatureType");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::FeatureWeight>("QDeclarativeGeoRouteQuery::FeatureWeight");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::RouteOptimization>("QDeclarativeGeoRouteQuery::RouteOptimization");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::RouteOptimizations>("QDeclarativeGeoRouteQuery::RouteOptimizations");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::RouteOptimization>("QDeclarativeGeoRouteQuery::RouteOptimization");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::SegmentDetail>("QDeclarativeGeoRouteQuery::SegmentDetail");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::SegmentDetails>("QDeclarativeGeoRouteQuery::SegmentDetails");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::SegmentDetail>("QDeclarativeGeoRouteQuery::SegmentDetail");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::ManeuverDetail>("QDeclarativeGeoRouteQuery::ManeuverDetail");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::ManeuverDetails>("QDeclarativeGeoRouteQuery::ManeuverDetails");
    qmlRegisterEnum<QDeclarativeGeoRouteQuery::ManeuverDetail>("QDeclarativeGeoRouteQuery::ManeuverDetail");
    qmlRegisterTypesAndRevisions<QDeclarativeGeoServiceProvider>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::RoutingFeature>("QDeclarativeGeoServiceProvider::RoutingFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::RoutingFeatures>("QDeclarativeGeoServiceProvider::RoutingFeatures");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::RoutingFeature>("QDeclarativeGeoServiceProvider::RoutingFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::GeocodingFeature>("QDeclarativeGeoServiceProvider::GeocodingFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::GeocodingFeatures>("QDeclarativeGeoServiceProvider::GeocodingFeatures");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::GeocodingFeature>("QDeclarativeGeoServiceProvider::GeocodingFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::MappingFeature>("QDeclarativeGeoServiceProvider::MappingFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::MappingFeatures>("QDeclarativeGeoServiceProvider::MappingFeatures");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::MappingFeature>("QDeclarativeGeoServiceProvider::MappingFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::PlacesFeature>("QDeclarativeGeoServiceProvider::PlacesFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::PlacesFeatures>("QDeclarativeGeoServiceProvider::PlacesFeatures");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::PlacesFeature>("QDeclarativeGeoServiceProvider::PlacesFeature");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::NavigationFeatures>("QDeclarativeGeoServiceProvider::NavigationFeatures");
    qmlRegisterEnum<QDeclarativeGeoServiceProvider::NavigationFeature>("QDeclarativeGeoServiceProvider::NavigationFeature");
    qmlRegisterTypesAndRevisions<QDeclarativeGeoServiceProviderRequirements>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeGeocodeModel>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativeGeocodeModel::Status>("QDeclarativeGeocodeModel::Status");
    qmlRegisterEnum<QDeclarativeGeocodeModel::GeocodeError>("QDeclarativeGeocodeModel::GeocodeError");
    qmlRegisterTypesAndRevisions<QDeclarativeMapLineProperties>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePlace>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativePlace::Status>("QDeclarativePlace::Status");
    qmlRegisterEnum<QDeclarativePlace::Visibility>("QDeclarativePlace::Visibility");
    QMetaType::fromType<QDeclarativePlaceContentModel *>().id();
    qmlRegisterTypesAndRevisions<QDeclarativePlaceEditorialModel>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePlaceImageModel>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePlaceReviewModel>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePluginParameterForeign>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePolygonMapItem>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativePolylineMapItem>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeRectangleMapItem>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeRouteMapItem>("QtLocation", 6);
    QMetaType::fromType<QDeclarativeSearchModelBase *>().id();
    qmlRegisterEnum<QDeclarativeSearchModelBase::Status>("QDeclarativeSearchModelBase::Status");
    qmlRegisterTypesAndRevisions<QDeclarativeSearchResultModel>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativeSearchResultModel::SearchResultType>("QDeclarativeSearchResultModel::SearchResultType");
    qmlRegisterEnum<QDeclarativeSearchResultModel::RelevanceHint>("QDeclarativeSearchResultModel::RelevanceHint");
    qmlRegisterTypesAndRevisions<QDeclarativeSearchSuggestionModel>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QDeclarativeSupportedCategoriesModel>("QtLocation", 6);
    qmlRegisterEnum<QDeclarativeSupportedCategoriesModel::Roles>("QDeclarativeSupportedCategoriesModel::Roles");
    qmlRegisterEnum<QDeclarativeSupportedCategoriesModel::Status>("QDeclarativeSupportedCategoriesModel::Status");
    qmlRegisterTypesAndRevisions<QGeoManeuver>("QtLocation", 6);
    qmlRegisterEnum<QGeoManeuver::InstructionDirection>("QGeoManeuver::InstructionDirection");
    QMetaType::fromType<QGeoManeuverDerived>().id();
    qmlRegisterNamespaceAndRevisions(&QGeoManeuverDerived::staticMetaObject, "QtLocation", 6, nullptr, &QGeoManeuverForeignNamespace::staticMetaObject, nullptr);
    qmlRegisterTypesAndRevisions<QGeoMapType>("QtLocation", 6);
    qmlRegisterEnum<QGeoMapType::MapStyle>("QGeoMapType::MapStyle");
    QMetaType::fromType<QGeoMapType>().id();
    qmlRegisterNamespaceAndRevisions(&QGeoMapType::staticMetaObject, "QtLocation", 6, nullptr, &QGeoMapTypeForeignNamespace::staticMetaObject, nullptr);
    qmlRegisterEnum<QGeoMapType::MapStyle>("QGeoMapType::MapStyle");
    qmlRegisterTypesAndRevisions<QGeoRoute>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QGeoRouteSegment>("QtLocation", 6);
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &QLocation::staticMetaObject;},
            "QLocation");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&QLocation::staticMetaObject, "QtLocation", 6, nullptr, &QLocation::staticMetaObject, nullptr);
    qmlRegisterEnum<QLocation::ReferenceSurface>("QLocation::ReferenceSurface");
    qmlRegisterTypesAndRevisions<QPlaceAttribute>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QPlaceContactDetail>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QPlaceIcon>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QPlaceRatings>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QPlaceSupplier>("QtLocation", 6);
    qmlRegisterTypesAndRevisions<QPlaceUser>("QtLocation", 6);
    QT_WARNING_POP
    qmlRegisterModule("QtLocation", 6, 11);
}

static const QQmlModuleRegistration qtLocationRegistration("QtLocation", qml_register_types_QtLocation);
