/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativesearchmodelbase_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativeplaces/qdeclarativesearchmodelbase_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativesearchmodelbase_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27QDeclarativeSearchModelBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeSearchModelBase::qt_create_metaobjectdata<qt_meta_tag_ZN27QDeclarativeSearchModelBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeSearchModelBase",
        "pluginChanged",
        "",
        "searchAreaChanged",
        "limitChanged",
        "previousPagesAvailableChanged",
        "nextPagesAvailableChanged",
        "statusChanged",
        "queryFinished",
        "onContentUpdated",
        "pluginNameChanged",
        "update",
        "cancel",
        "reset",
        "errorString",
        "previousPage",
        "nextPage",
        "plugin",
        "QDeclarativeGeoServiceProvider*",
        "searchArea",
        "QVariant",
        "limit",
        "previousPagesAvailable",
        "nextPagesAvailable",
        "status",
        "Status",
        "Null",
        "Ready",
        "Loading",
        "Error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pluginChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'searchAreaChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'limitChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previousPagesAvailableChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nextPagesAvailableChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'queryFinished'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onContentUpdated'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'pluginNameChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'update'
        QtMocHelpers::MethodData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'cancel'
        QtMocHelpers::MethodData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'reset'
        QtMocHelpers::MethodData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'errorString'
        QtMocHelpers::MethodData<QString() const>(14, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'previousPage'
        QtMocHelpers::MethodData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'nextPage'
        QtMocHelpers::MethodData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'plugin'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider*>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'searchArea'
        QtMocHelpers::PropertyData<QVariant>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'limit'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'previousPagesAvailable'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'nextPagesAvailable'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(25, 25, QMC::EnumFlags{}).add({
            {   26, Status::Null },
            {   27, Status::Ready },
            {   28, Status::Loading },
            {   29, Status::Error },
        }),
    };
    return QtMocHelpers::metaObjectData<QDeclarativeSearchModelBase, qt_meta_tag_ZN27QDeclarativeSearchModelBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDeclarativeSearchModelBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeSearchModelBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeSearchModelBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QDeclarativeSearchModelBaseE_t>.metaTypes,
    nullptr
} };

void QDeclarativeSearchModelBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeSearchModelBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pluginChanged(); break;
        case 1: _t->searchAreaChanged(); break;
        case 2: _t->limitChanged(); break;
        case 3: _t->previousPagesAvailableChanged(); break;
        case 4: _t->nextPagesAvailableChanged(); break;
        case 5: _t->statusChanged(); break;
        case 6: _t->queryFinished(); break;
        case 7: _t->onContentUpdated(); break;
        case 8: _t->pluginNameChanged(); break;
        case 9: _t->update(); break;
        case 10: _t->cancel(); break;
        case 11: _t->reset(); break;
        case 12: { QString _r = _t->errorString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->previousPage(); break;
        case 14: _t->nextPage(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchModelBase::*)()>(_a, &QDeclarativeSearchModelBase::pluginChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchModelBase::*)()>(_a, &QDeclarativeSearchModelBase::searchAreaChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchModelBase::*)()>(_a, &QDeclarativeSearchModelBase::limitChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchModelBase::*)()>(_a, &QDeclarativeSearchModelBase::previousPagesAvailableChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchModelBase::*)()>(_a, &QDeclarativeSearchModelBase::nextPagesAvailableChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchModelBase::*)()>(_a, &QDeclarativeSearchModelBase::statusChanged, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoServiceProvider* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v) = _t->plugin(); break;
        case 1: *reinterpret_cast<QVariant*>(_v) = _t->searchArea(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->limit(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->previousPagesAvailable(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->nextPagesAvailable(); break;
        case 5: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlugin(*reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v)); break;
        case 1: _t->setSearchArea(*reinterpret_cast<QVariant*>(_v)); break;
        case 2: _t->setLimit(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeSearchModelBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeSearchModelBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeSearchModelBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QDeclarativeSearchModelBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeSearchModelBase::pluginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeSearchModelBase::searchAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeSearchModelBase::limitChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeSearchModelBase::previousPagesAvailableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeSearchModelBase::nextPagesAvailableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeSearchModelBase::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
