/****************************************************************************
** Meta object code from reading C++ file 'qgeoserviceprovider.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/maps/qgeoserviceprovider.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeoserviceprovider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19QGeoServiceProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoServiceProvider::qt_create_metaobjectdata<qt_meta_tag_ZN19QGeoServiceProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoServiceProvider",
        "Error",
        "NoError",
        "NotSupportedError",
        "UnknownParameterError",
        "MissingRequiredParameterError",
        "ConnectionError",
        "LoaderError",
        "RoutingFeatures",
        "RoutingFeature",
        "NoRoutingFeatures",
        "OnlineRoutingFeature",
        "OfflineRoutingFeature",
        "LocalizedRoutingFeature",
        "RouteUpdatesFeature",
        "AlternativeRoutesFeature",
        "ExcludeAreasRoutingFeature",
        "AnyRoutingFeatures",
        "GeocodingFeatures",
        "GeocodingFeature",
        "NoGeocodingFeatures",
        "OnlineGeocodingFeature",
        "OfflineGeocodingFeature",
        "ReverseGeocodingFeature",
        "LocalizedGeocodingFeature",
        "AnyGeocodingFeatures",
        "MappingFeatures",
        "MappingFeature",
        "NoMappingFeatures",
        "OnlineMappingFeature",
        "OfflineMappingFeature",
        "LocalizedMappingFeature",
        "AnyMappingFeatures",
        "PlacesFeatures",
        "PlacesFeature",
        "NoPlacesFeatures",
        "OnlinePlacesFeature",
        "OfflinePlacesFeature",
        "SavePlaceFeature",
        "RemovePlaceFeature",
        "SaveCategoryFeature",
        "RemoveCategoryFeature",
        "PlaceRecommendationsFeature",
        "SearchSuggestionsFeature",
        "LocalizedPlacesFeature",
        "NotificationsFeature",
        "PlaceMatchingFeature",
        "AnyPlacesFeatures",
        "NavigationFeatures",
        "NavigationFeature",
        "NoNavigationFeatures",
        "OnlineNavigationFeature",
        "OfflineNavigationFeature",
        "AnyNavigationFeatures"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(1, 1, QMC::EnumFlags{}).add({
            {    2, Error::NoError },
            {    3, Error::NotSupportedError },
            {    4, Error::UnknownParameterError },
            {    5, Error::MissingRequiredParameterError },
            {    6, Error::ConnectionError },
            {    7, Error::LoaderError },
        }),
        // flag 'RoutingFeatures'
        QtMocHelpers::EnumData<RoutingFeatures>(8, 9, QMC::EnumIsFlag).add({
            {   10, RoutingFeature::NoRoutingFeatures },
            {   11, RoutingFeature::OnlineRoutingFeature },
            {   12, RoutingFeature::OfflineRoutingFeature },
            {   13, RoutingFeature::LocalizedRoutingFeature },
            {   14, RoutingFeature::RouteUpdatesFeature },
            {   15, RoutingFeature::AlternativeRoutesFeature },
            {   16, RoutingFeature::ExcludeAreasRoutingFeature },
            {   17, RoutingFeature::AnyRoutingFeatures },
        }),
        // flag 'GeocodingFeatures'
        QtMocHelpers::EnumData<GeocodingFeatures>(18, 19, QMC::EnumIsFlag).add({
            {   20, GeocodingFeature::NoGeocodingFeatures },
            {   21, GeocodingFeature::OnlineGeocodingFeature },
            {   22, GeocodingFeature::OfflineGeocodingFeature },
            {   23, GeocodingFeature::ReverseGeocodingFeature },
            {   24, GeocodingFeature::LocalizedGeocodingFeature },
            {   25, GeocodingFeature::AnyGeocodingFeatures },
        }),
        // flag 'MappingFeatures'
        QtMocHelpers::EnumData<MappingFeatures>(26, 27, QMC::EnumIsFlag).add({
            {   28, MappingFeature::NoMappingFeatures },
            {   29, MappingFeature::OnlineMappingFeature },
            {   30, MappingFeature::OfflineMappingFeature },
            {   31, MappingFeature::LocalizedMappingFeature },
            {   32, MappingFeature::AnyMappingFeatures },
        }),
        // flag 'PlacesFeatures'
        QtMocHelpers::EnumData<PlacesFeatures>(33, 34, QMC::EnumIsFlag).add({
            {   35, PlacesFeature::NoPlacesFeatures },
            {   36, PlacesFeature::OnlinePlacesFeature },
            {   37, PlacesFeature::OfflinePlacesFeature },
            {   38, PlacesFeature::SavePlaceFeature },
            {   39, PlacesFeature::RemovePlaceFeature },
            {   40, PlacesFeature::SaveCategoryFeature },
            {   41, PlacesFeature::RemoveCategoryFeature },
            {   42, PlacesFeature::PlaceRecommendationsFeature },
            {   43, PlacesFeature::SearchSuggestionsFeature },
            {   44, PlacesFeature::LocalizedPlacesFeature },
            {   45, PlacesFeature::NotificationsFeature },
            {   46, PlacesFeature::PlaceMatchingFeature },
            {   47, PlacesFeature::AnyPlacesFeatures },
        }),
        // flag 'NavigationFeatures'
        QtMocHelpers::EnumData<NavigationFeatures>(48, 49, QMC::EnumIsFlag).add({
            {   50, NavigationFeature::NoNavigationFeatures },
            {   51, NavigationFeature::OnlineNavigationFeature },
            {   52, NavigationFeature::OfflineNavigationFeature },
            {   53, NavigationFeature::AnyNavigationFeatures },
        }),
    };
    return QtMocHelpers::metaObjectData<QGeoServiceProvider, qt_meta_tag_ZN19QGeoServiceProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoServiceProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoServiceProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoServiceProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QGeoServiceProviderE_t>.metaTypes,
    nullptr
} };

void QGeoServiceProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGeoServiceProvider *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QGeoServiceProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGeoServiceProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoServiceProviderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QGeoServiceProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
