/****************************************************************************
** Meta object code from reading C++ file 'qplacemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/places/qplacemanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qplacemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QPlaceManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceManager::qt_create_metaobjectdata<qt_meta_tag_ZN13QPlaceManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceManager",
        "finished",
        "",
        "QPlaceReply*",
        "reply",
        "errorOccurred",
        "QPlaceReply::Error",
        "error",
        "errorString",
        "placeAdded",
        "placeId",
        "placeUpdated",
        "placeRemoved",
        "categoryAdded",
        "QPlaceCategory",
        "category",
        "parentId",
        "categoryUpdated",
        "categoryRemoved",
        "categoryId",
        "dataChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void(QPlaceReply *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QPlaceReply *, QPlaceReply::Error, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 6, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QPlaceReply *, QPlaceReply::Error)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'placeAdded'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'placeUpdated'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'placeRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'categoryAdded'
        QtMocHelpers::SignalData<void(const QPlaceCategory &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 },
        }}),
        // Signal 'categoryUpdated'
        QtMocHelpers::SignalData<void(const QPlaceCategory &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 },
        }}),
        // Signal 'categoryRemoved'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 }, { QMetaType::QString, 16 },
        }}),
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceManager, qt_meta_tag_ZN13QPlaceManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QPlaceManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QPlaceManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QPlaceManagerE_t>.metaTypes,
    nullptr
} };

void QPlaceManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished((*reinterpret_cast<std::add_pointer_t<QPlaceReply*>>(_a[1]))); break;
        case 1: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QPlaceReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPlaceReply::Error>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QPlaceReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPlaceReply::Error>>(_a[2]))); break;
        case 3: _t->placeAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->placeUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->placeRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->categoryAdded((*reinterpret_cast<std::add_pointer_t<QPlaceCategory>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->categoryUpdated((*reinterpret_cast<std::add_pointer_t<QPlaceCategory>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->categoryRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->dataChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPlaceReply* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPlaceReply* >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPlaceReply::Error >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPlaceReply* >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPlaceReply::Error >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(QPlaceReply * )>(_a, &QPlaceManager::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(QPlaceReply * , QPlaceReply::Error , const QString & )>(_a, &QPlaceManager::errorOccurred, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(const QString & )>(_a, &QPlaceManager::placeAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(const QString & )>(_a, &QPlaceManager::placeUpdated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(const QString & )>(_a, &QPlaceManager::placeRemoved, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(const QPlaceCategory & , const QString & )>(_a, &QPlaceManager::categoryAdded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(const QPlaceCategory & , const QString & )>(_a, &QPlaceManager::categoryUpdated, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)(const QString & , const QString & )>(_a, &QPlaceManager::categoryRemoved, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPlaceManager::*)()>(_a, &QPlaceManager::dataChanged, 9))
            return;
    }
}

const QMetaObject *QPlaceManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QPlaceManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QPlaceManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QPlaceManager::finished(QPlaceReply * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QPlaceManager::errorOccurred(QPlaceReply * _t1, QPlaceReply::Error _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void QPlaceManager::placeAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QPlaceManager::placeUpdated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QPlaceManager::placeRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QPlaceManager::categoryAdded(const QPlaceCategory & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void QPlaceManager::categoryUpdated(const QPlaceCategory & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void QPlaceManager::categoryRemoved(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void QPlaceManager::dataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
