/****************************************************************************
** Meta object code from reading C++ file 'qgeomaneuver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/maps/qgeomaneuver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeomaneuver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12QGeoManeuverE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoManeuver::qt_create_metaobjectdata<qt_meta_tag_ZN12QGeoManeuverE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoManeuver",
        "QML.Element",
        "routeManeuver",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "valid",
        "position",
        "QGeoCoordinate",
        "instructionText",
        "direction",
        "InstructionDirection",
        "timeToNextInstruction",
        "distanceToNextInstruction",
        "waypoint",
        "extendedAttributes",
        "QVariantMap",
        "NoDirection",
        "DirectionForward",
        "DirectionBearRight",
        "DirectionLightRight",
        "DirectionRight",
        "DirectionHardRight",
        "DirectionUTurnRight",
        "DirectionUTurnLeft",
        "DirectionHardLeft",
        "DirectionLeft",
        "DirectionLightLeft",
        "DirectionBearLeft"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'valid'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'position'
        QtMocHelpers::PropertyData<QGeoCoordinate>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'instructionText'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'direction'
        QtMocHelpers::PropertyData<enum InstructionDirection>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'timeToNextInstruction'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'distanceToNextInstruction'
        QtMocHelpers::PropertyData<qreal>(14, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'waypoint'
        QtMocHelpers::PropertyData<QGeoCoordinate>(15, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extendedAttributes'
        QtMocHelpers::PropertyData<QVariantMap>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'InstructionDirection'
        QtMocHelpers::EnumData<enum InstructionDirection>(12, 12, QMC::EnumFlags{}).add({
            {   18, InstructionDirection::NoDirection },
            {   19, InstructionDirection::DirectionForward },
            {   20, InstructionDirection::DirectionBearRight },
            {   21, InstructionDirection::DirectionLightRight },
            {   22, InstructionDirection::DirectionRight },
            {   23, InstructionDirection::DirectionHardRight },
            {   24, InstructionDirection::DirectionUTurnRight },
            {   25, InstructionDirection::DirectionUTurnLeft },
            {   26, InstructionDirection::DirectionHardLeft },
            {   27, InstructionDirection::DirectionLeft },
            {   28, InstructionDirection::DirectionLightLeft },
            {   29, InstructionDirection::DirectionBearLeft },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QGeoManeuver, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoManeuver::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QGeoManeuverE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QGeoManeuverE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QGeoManeuverE_t>.metaTypes,
    nullptr
} };

void QGeoManeuver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoManeuver *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        case 1: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->position(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->instructionText(); break;
        case 3: *reinterpret_cast<enum InstructionDirection*>(_v) = _t->direction(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->timeToNextInstruction(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->distanceToNextInstruction(); break;
        case 6: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->waypoint(); break;
        case 7: *reinterpret_cast<QVariantMap*>(_v) = _t->extendedAttributes(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
