/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativeplacecontentmodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativeplaces/qdeclarativeplacecontentmodel_p.h"
#include <QtLocation/private/qdeclarativeplace_p.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativeplacecontentmodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29QDeclarativePlaceContentModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePlaceContentModel::qt_create_metaobjectdata<qt_meta_tag_ZN29QDeclarativePlaceContentModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePlaceContentModel",
        "placeChanged",
        "",
        "batchSizeChanged",
        "totalCountChanged",
        "fetchFinished",
        "place",
        "QDeclarativePlace*",
        "batchSize",
        "totalCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'placeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'batchSizeChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'totalCountChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchFinished'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'place'
        QtMocHelpers::PropertyData<QDeclarativePlace*>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'batchSize'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'totalCount'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDeclarativePlaceContentModel, qt_meta_tag_ZN29QDeclarativePlaceContentModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDeclarativePlaceContentModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativePlaceContentModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativePlaceContentModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QDeclarativePlaceContentModelE_t>.metaTypes,
    nullptr
} };

void QDeclarativePlaceContentModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativePlaceContentModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->placeChanged(); break;
        case 1: _t->batchSizeChanged(); break;
        case 2: _t->totalCountChanged(); break;
        case 3: _t->fetchFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlaceContentModel::*)()>(_a, &QDeclarativePlaceContentModel::placeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlaceContentModel::*)()>(_a, &QDeclarativePlaceContentModel::batchSizeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlaceContentModel::*)()>(_a, &QDeclarativePlaceContentModel::totalCountChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativePlace**>(_v) = _t->place(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->batchSize(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->totalCount(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlace(*reinterpret_cast<QDeclarativePlace**>(_v)); break;
        case 1: _t->setBatchSize(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativePlaceContentModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativePlaceContentModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativePlaceContentModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QDeclarativePlaceContentModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativePlaceContentModel::placeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativePlaceContentModel::batchSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativePlaceContentModel::totalCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN28QDeclarativePlaceReviewModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePlaceReviewModel::qt_create_metaobjectdata<qt_meta_tag_ZN28QDeclarativePlaceReviewModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePlaceReviewModel",
        "QML.Element",
        "ReviewModel",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativePlaceReviewModel, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativePlaceReviewModel::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QDeclarativePlaceContentModel>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDeclarativePlaceReviewModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDeclarativePlaceReviewModelE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QDeclarativePlaceReviewModelE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN31QDeclarativePlaceEditorialModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePlaceEditorialModel::qt_create_metaobjectdata<qt_meta_tag_ZN31QDeclarativePlaceEditorialModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePlaceEditorialModel",
        "QML.Element",
        "EditorialModel",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativePlaceEditorialModel, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativePlaceEditorialModel::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QDeclarativePlaceContentModel>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QDeclarativePlaceEditorialModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QDeclarativePlaceEditorialModelE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QDeclarativePlaceEditorialModelE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN27QDeclarativePlaceImageModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePlaceImageModel::qt_create_metaobjectdata<qt_meta_tag_ZN27QDeclarativePlaceImageModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePlaceImageModel",
        "QML.Element",
        "ImageModel",
        "QML.AddedInVersion",
        "1280"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativePlaceImageModel, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativePlaceImageModel::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QDeclarativePlaceContentModel>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativePlaceImageModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativePlaceImageModelE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QDeclarativePlaceImageModelE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
