/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeoserviceprovider_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativemaps/qdeclarativegeoserviceprovider_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeoserviceprovider_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN34QDeclarativePluginParameterForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePluginParameterForeign::qt_create_metaobjectdata<qt_meta_tag_ZN34QDeclarativePluginParameterForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePluginParameterForeign",
        "QML.Foreign",
        "QDeclarativePluginParameter",
        "QML.Element",
        "PluginParameter"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativePluginParameterForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativePluginParameterForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34QDeclarativePluginParameterForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34QDeclarativePluginParameterForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34QDeclarativePluginParameterForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN30QDeclarativeGeoServiceProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoServiceProvider::qt_create_metaobjectdata<qt_meta_tag_ZN30QDeclarativeGeoServiceProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoServiceProvider",
        "QML.Element",
        "Plugin",
        "QML.AddedInVersion",
        "1280",
        "DefaultProperty",
        "parameters",
        "nameChanged",
        "",
        "name",
        "localesChanged",
        "attached",
        "preferredChanged",
        "preferences",
        "allowExperimentalChanged",
        "allow",
        "supportsRouting",
        "RoutingFeatures",
        "feature",
        "supportsGeocoding",
        "GeocodingFeatures",
        "supportsMapping",
        "MappingFeatures",
        "supportsPlaces",
        "PlacesFeatures",
        "supportsNavigation",
        "NavigationFeature",
        "availableServiceProviders",
        "QQmlListProperty<QDeclarativePluginParameter>",
        "required",
        "QDeclarativeGeoServiceProviderRequirements*",
        "locales",
        "preferred",
        "allowExperimental",
        "isAttached",
        "RoutingFeature",
        "NoRoutingFeatures",
        "OnlineRoutingFeature",
        "OfflineRoutingFeature",
        "LocalizedRoutingFeature",
        "RouteUpdatesFeature",
        "AlternativeRoutesFeature",
        "ExcludeAreasRoutingFeature",
        "AnyRoutingFeatures",
        "GeocodingFeature",
        "NoGeocodingFeatures",
        "OnlineGeocodingFeature",
        "OfflineGeocodingFeature",
        "ReverseGeocodingFeature",
        "LocalizedGeocodingFeature",
        "AnyGeocodingFeatures",
        "MappingFeature",
        "NoMappingFeatures",
        "OnlineMappingFeature",
        "OfflineMappingFeature",
        "LocalizedMappingFeature",
        "AnyMappingFeatures",
        "PlacesFeature",
        "NoPlacesFeatures",
        "OnlinePlacesFeature",
        "OfflinePlacesFeature",
        "SavePlaceFeature",
        "RemovePlaceFeature",
        "SaveCategoryFeature",
        "RemoveCategoryFeature",
        "PlaceRecommendationsFeature",
        "SearchSuggestionsFeature",
        "LocalizedPlacesFeature",
        "NotificationsFeature",
        "PlaceMatchingFeature",
        "AnyPlacesFeatures",
        "NavigationFeatures",
        "NoNavigationFeatures",
        "OnlineNavigationFeature",
        "OfflineNavigationFeature",
        "AnyNavigationFeatures"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'localesChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'attached'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preferredChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 13 },
        }}),
        // Signal 'allowExperimentalChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Method 'supportsRouting'
        QtMocHelpers::MethodData<bool(const RoutingFeatures &) const>(16, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'supportsRouting'
        QtMocHelpers::MethodData<bool() const>(16, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Method 'supportsGeocoding'
        QtMocHelpers::MethodData<bool(const GeocodingFeatures &) const>(19, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 18 },
        }}),
        // Method 'supportsGeocoding'
        QtMocHelpers::MethodData<bool() const>(19, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Method 'supportsMapping'
        QtMocHelpers::MethodData<bool(const MappingFeatures &) const>(21, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 22, 18 },
        }}),
        // Method 'supportsMapping'
        QtMocHelpers::MethodData<bool() const>(21, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Method 'supportsPlaces'
        QtMocHelpers::MethodData<bool(const PlacesFeatures &) const>(23, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 24, 18 },
        }}),
        // Method 'supportsPlaces'
        QtMocHelpers::MethodData<bool() const>(23, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Method 'supportsNavigation'
        QtMocHelpers::RevisionedMethodData<bool(const NavigationFeature &) const>(25, 8, QMC::AccessPublic, 0x50b, QMetaType::Bool, {{
            { 0x80000000 | 26, 18 },
        }}),
        // Method 'supportsNavigation'
        QtMocHelpers::RevisionedMethodData<bool() const>(25, 8, QMC::AccessPublic | QMC::MethodCloned, 0x50b, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'availableServiceProviders'
        QtMocHelpers::PropertyData<QStringList>(27, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'parameters'
        QtMocHelpers::PropertyData<QQmlListProperty<QDeclarativePluginParameter>>(6, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'required'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProviderRequirements*>(29, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'locales'
        QtMocHelpers::PropertyData<QStringList>(31, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'preferred'
        QtMocHelpers::PropertyData<QStringList>(32, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'allowExperimental'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isAttached'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'RoutingFeature'
        QtMocHelpers::EnumData<enum RoutingFeature>(35, 35, QMC::EnumFlags{}).add({
            {   36, RoutingFeature::NoRoutingFeatures },
            {   37, RoutingFeature::OnlineRoutingFeature },
            {   38, RoutingFeature::OfflineRoutingFeature },
            {   39, RoutingFeature::LocalizedRoutingFeature },
            {   40, RoutingFeature::RouteUpdatesFeature },
            {   41, RoutingFeature::AlternativeRoutesFeature },
            {   42, RoutingFeature::ExcludeAreasRoutingFeature },
            {   43, RoutingFeature::AnyRoutingFeatures },
        }),
        // flag 'RoutingFeatures'
        QtMocHelpers::EnumData<RoutingFeatures>(17, 35, QMC::EnumIsFlag).add({
            {   36, RoutingFeature::NoRoutingFeatures },
            {   37, RoutingFeature::OnlineRoutingFeature },
            {   38, RoutingFeature::OfflineRoutingFeature },
            {   39, RoutingFeature::LocalizedRoutingFeature },
            {   40, RoutingFeature::RouteUpdatesFeature },
            {   41, RoutingFeature::AlternativeRoutesFeature },
            {   42, RoutingFeature::ExcludeAreasRoutingFeature },
            {   43, RoutingFeature::AnyRoutingFeatures },
        }),
        // enum 'GeocodingFeature'
        QtMocHelpers::EnumData<enum GeocodingFeature>(44, 44, QMC::EnumFlags{}).add({
            {   45, GeocodingFeature::NoGeocodingFeatures },
            {   46, GeocodingFeature::OnlineGeocodingFeature },
            {   47, GeocodingFeature::OfflineGeocodingFeature },
            {   48, GeocodingFeature::ReverseGeocodingFeature },
            {   49, GeocodingFeature::LocalizedGeocodingFeature },
            {   50, GeocodingFeature::AnyGeocodingFeatures },
        }),
        // flag 'GeocodingFeatures'
        QtMocHelpers::EnumData<GeocodingFeatures>(20, 44, QMC::EnumIsFlag).add({
            {   45, GeocodingFeature::NoGeocodingFeatures },
            {   46, GeocodingFeature::OnlineGeocodingFeature },
            {   47, GeocodingFeature::OfflineGeocodingFeature },
            {   48, GeocodingFeature::ReverseGeocodingFeature },
            {   49, GeocodingFeature::LocalizedGeocodingFeature },
            {   50, GeocodingFeature::AnyGeocodingFeatures },
        }),
        // enum 'MappingFeature'
        QtMocHelpers::EnumData<enum MappingFeature>(51, 51, QMC::EnumFlags{}).add({
            {   52, MappingFeature::NoMappingFeatures },
            {   53, MappingFeature::OnlineMappingFeature },
            {   54, MappingFeature::OfflineMappingFeature },
            {   55, MappingFeature::LocalizedMappingFeature },
            {   56, MappingFeature::AnyMappingFeatures },
        }),
        // flag 'MappingFeatures'
        QtMocHelpers::EnumData<MappingFeatures>(22, 51, QMC::EnumIsFlag).add({
            {   52, MappingFeature::NoMappingFeatures },
            {   53, MappingFeature::OnlineMappingFeature },
            {   54, MappingFeature::OfflineMappingFeature },
            {   55, MappingFeature::LocalizedMappingFeature },
            {   56, MappingFeature::AnyMappingFeatures },
        }),
        // enum 'PlacesFeature'
        QtMocHelpers::EnumData<enum PlacesFeature>(57, 57, QMC::EnumFlags{}).add({
            {   58, PlacesFeature::NoPlacesFeatures },
            {   59, PlacesFeature::OnlinePlacesFeature },
            {   60, PlacesFeature::OfflinePlacesFeature },
            {   61, PlacesFeature::SavePlaceFeature },
            {   62, PlacesFeature::RemovePlaceFeature },
            {   63, PlacesFeature::SaveCategoryFeature },
            {   64, PlacesFeature::RemoveCategoryFeature },
            {   65, PlacesFeature::PlaceRecommendationsFeature },
            {   66, PlacesFeature::SearchSuggestionsFeature },
            {   67, PlacesFeature::LocalizedPlacesFeature },
            {   68, PlacesFeature::NotificationsFeature },
            {   69, PlacesFeature::PlaceMatchingFeature },
            {   70, PlacesFeature::AnyPlacesFeatures },
        }),
        // flag 'PlacesFeatures'
        QtMocHelpers::EnumData<PlacesFeatures>(24, 57, QMC::EnumIsFlag).add({
            {   58, PlacesFeature::NoPlacesFeatures },
            {   59, PlacesFeature::OnlinePlacesFeature },
            {   60, PlacesFeature::OfflinePlacesFeature },
            {   61, PlacesFeature::SavePlaceFeature },
            {   62, PlacesFeature::RemovePlaceFeature },
            {   63, PlacesFeature::SaveCategoryFeature },
            {   64, PlacesFeature::RemoveCategoryFeature },
            {   65, PlacesFeature::PlaceRecommendationsFeature },
            {   66, PlacesFeature::SearchSuggestionsFeature },
            {   67, PlacesFeature::LocalizedPlacesFeature },
            {   68, PlacesFeature::NotificationsFeature },
            {   69, PlacesFeature::PlaceMatchingFeature },
            {   70, PlacesFeature::AnyPlacesFeatures },
        }),
        // flag 'NavigationFeatures'
        QtMocHelpers::EnumData<NavigationFeatures>(71, 26, QMC::EnumIsFlag).add({
            {   72, NavigationFeature::NoNavigationFeatures },
            {   73, NavigationFeature::OnlineNavigationFeature },
            {   74, NavigationFeature::OfflineNavigationFeature },
            {   75, NavigationFeature::AnyNavigationFeatures },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoServiceProvider, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeoServiceProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QDeclarativeGeoServiceProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QDeclarativeGeoServiceProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30QDeclarativeGeoServiceProviderE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoServiceProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoServiceProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->localesChanged(); break;
        case 2: _t->attached(); break;
        case 3: _t->preferredChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->allowExperimentalChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: { bool _r = _t->supportsRouting((*reinterpret_cast<std::add_pointer_t<RoutingFeatures>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->supportsRouting();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->supportsGeocoding((*reinterpret_cast<std::add_pointer_t<GeocodingFeatures>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->supportsGeocoding();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->supportsMapping((*reinterpret_cast<std::add_pointer_t<MappingFeatures>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->supportsMapping();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->supportsPlaces((*reinterpret_cast<std::add_pointer_t<PlacesFeatures>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->supportsPlaces();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->supportsNavigation((*reinterpret_cast<std::add_pointer_t<enum NavigationFeature>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->supportsNavigation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProvider::*)(const QString & )>(_a, &QDeclarativeGeoServiceProvider::nameChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProvider::*)()>(_a, &QDeclarativeGeoServiceProvider::localesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProvider::*)()>(_a, &QDeclarativeGeoServiceProvider::attached, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProvider::*)(const QStringList & )>(_a, &QDeclarativeGeoServiceProvider::preferredChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProvider::*)(bool )>(_a, &QDeclarativeGeoServiceProvider::allowExperimentalChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoServiceProviderRequirements* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativePluginParameter> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->availableServiceProviders(); break;
        case 2: *reinterpret_cast<QQmlListProperty<QDeclarativePluginParameter>*>(_v) = _t->parameters(); break;
        case 3: *reinterpret_cast<QDeclarativeGeoServiceProviderRequirements**>(_v) = _t->requirements(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->locales(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->preferred(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->allowExperimental(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isAttached(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setRequirements(*reinterpret_cast<QDeclarativeGeoServiceProviderRequirements**>(_v)); break;
        case 4: _t->setLocales(*reinterpret_cast<QStringList*>(_v)); break;
        case 5: _t->setPreferred(*reinterpret_cast<QStringList*>(_v)); break;
        case 6: _t->setAllowExperimental(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoServiceProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoServiceProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QDeclarativeGeoServiceProviderE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeGeoServiceProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoServiceProvider::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDeclarativeGeoServiceProvider::localesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoServiceProvider::attached()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeoServiceProvider::preferredChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QDeclarativeGeoServiceProvider::allowExperimentalChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN42QDeclarativeGeoServiceProviderRequirementsE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoServiceProviderRequirements::qt_create_metaobjectdata<qt_meta_tag_ZN42QDeclarativeGeoServiceProviderRequirementsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoServiceProviderRequirements",
        "QML.Element",
        "PluginRequirements",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "PluginRequirements is not intended instantiable by developer.",
        "QML.AddedInVersion",
        "1280",
        "mappingRequirementsChanged",
        "",
        "QDeclarativeGeoServiceProvider::MappingFeatures",
        "features",
        "routingRequirementsChanged",
        "QDeclarativeGeoServiceProvider::RoutingFeatures",
        "geocodingRequirementsChanged",
        "QDeclarativeGeoServiceProvider::GeocodingFeatures",
        "placesRequirementsChanged",
        "QDeclarativeGeoServiceProvider::PlacesFeatures",
        "navigationRequirementsChanged",
        "QDeclarativeGeoServiceProvider::NavigationFeatures",
        "requirementsChanged",
        "matches",
        "const QGeoServiceProvider*",
        "provider",
        "mapping",
        "routing",
        "geocoding",
        "places",
        "navigation"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mappingRequirementsChanged'
        QtMocHelpers::SignalData<void(const QDeclarativeGeoServiceProvider::MappingFeatures &)>(9, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'routingRequirementsChanged'
        QtMocHelpers::SignalData<void(const QDeclarativeGeoServiceProvider::RoutingFeatures &)>(13, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 12 },
        }}),
        // Signal 'geocodingRequirementsChanged'
        QtMocHelpers::SignalData<void(const QDeclarativeGeoServiceProvider::GeocodingFeatures &)>(15, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 12 },
        }}),
        // Signal 'placesRequirementsChanged'
        QtMocHelpers::SignalData<void(const QDeclarativeGeoServiceProvider::PlacesFeatures &)>(17, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 12 },
        }}),
        // Signal 'navigationRequirementsChanged'
        QtMocHelpers::SignalData<void(const QDeclarativeGeoServiceProvider::NavigationFeatures &)>(19, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 12 },
        }}),
        // Signal 'requirementsChanged'
        QtMocHelpers::SignalData<void()>(21, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'matches'
        QtMocHelpers::MethodData<bool(const QGeoServiceProvider *) const>(22, 10, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'mapping'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider::MappingFeatures>(25, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 0),
        // property 'routing'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider::RoutingFeatures>(26, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 1),
        // property 'geocoding'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider::GeocodingFeatures>(27, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 2),
        // property 'places'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider::PlacesFeatures>(28, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 3),
        // property 'navigation'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider::NavigationFeatures>(29, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoServiceProviderRequirements, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN42QDeclarativeGeoServiceProviderRequirementsE[] = {
    QMetaObject::SuperData::link<QDeclarativeGeoServiceProvider::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QDeclarativeGeoServiceProviderRequirements::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QDeclarativeGeoServiceProviderRequirementsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QDeclarativeGeoServiceProviderRequirementsE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN42QDeclarativeGeoServiceProviderRequirementsE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN42QDeclarativeGeoServiceProviderRequirementsE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoServiceProviderRequirements::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoServiceProviderRequirements *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mappingRequirementsChanged((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoServiceProvider::MappingFeatures>>(_a[1]))); break;
        case 1: _t->routingRequirementsChanged((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoServiceProvider::RoutingFeatures>>(_a[1]))); break;
        case 2: _t->geocodingRequirementsChanged((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoServiceProvider::GeocodingFeatures>>(_a[1]))); break;
        case 3: _t->placesRequirementsChanged((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoServiceProvider::PlacesFeatures>>(_a[1]))); break;
        case 4: _t->navigationRequirementsChanged((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoServiceProvider::NavigationFeatures>>(_a[1]))); break;
        case 5: _t->requirementsChanged(); break;
        case 6: { bool _r = _t->matches((*reinterpret_cast<std::add_pointer_t<const QGeoServiceProvider*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProviderRequirements::*)(const QDeclarativeGeoServiceProvider::MappingFeatures & )>(_a, &QDeclarativeGeoServiceProviderRequirements::mappingRequirementsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProviderRequirements::*)(const QDeclarativeGeoServiceProvider::RoutingFeatures & )>(_a, &QDeclarativeGeoServiceProviderRequirements::routingRequirementsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProviderRequirements::*)(const QDeclarativeGeoServiceProvider::GeocodingFeatures & )>(_a, &QDeclarativeGeoServiceProviderRequirements::geocodingRequirementsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProviderRequirements::*)(const QDeclarativeGeoServiceProvider::PlacesFeatures & )>(_a, &QDeclarativeGeoServiceProviderRequirements::placesRequirementsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProviderRequirements::*)(const QDeclarativeGeoServiceProvider::NavigationFeatures & )>(_a, &QDeclarativeGeoServiceProviderRequirements::navigationRequirementsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoServiceProviderRequirements::*)()>(_a, &QDeclarativeGeoServiceProviderRequirements::requirementsChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativeGeoServiceProvider::MappingFeatures*>(_v) = _t->mappingRequirements(); break;
        case 1: *reinterpret_cast<QDeclarativeGeoServiceProvider::RoutingFeatures*>(_v) = _t->routingRequirements(); break;
        case 2: *reinterpret_cast<QDeclarativeGeoServiceProvider::GeocodingFeatures*>(_v) = _t->geocodingRequirements(); break;
        case 3: *reinterpret_cast<QDeclarativeGeoServiceProvider::PlacesFeatures*>(_v) = _t->placesRequirements(); break;
        case 4: *reinterpret_cast<QDeclarativeGeoServiceProvider::NavigationFeatures*>(_v) = _t->navigationRequirements(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMappingRequirements(*reinterpret_cast<QDeclarativeGeoServiceProvider::MappingFeatures*>(_v)); break;
        case 1: _t->setRoutingRequirements(*reinterpret_cast<QDeclarativeGeoServiceProvider::RoutingFeatures*>(_v)); break;
        case 2: _t->setGeocodingRequirements(*reinterpret_cast<QDeclarativeGeoServiceProvider::GeocodingFeatures*>(_v)); break;
        case 3: _t->setPlacesRequirements(*reinterpret_cast<QDeclarativeGeoServiceProvider::PlacesFeatures*>(_v)); break;
        case 4: _t->setNavigationRequirements(*reinterpret_cast<QDeclarativeGeoServiceProvider::NavigationFeatures*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoServiceProviderRequirements::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoServiceProviderRequirements::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QDeclarativeGeoServiceProviderRequirementsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeGeoServiceProviderRequirements::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoServiceProviderRequirements::mappingRequirementsChanged(const QDeclarativeGeoServiceProvider::MappingFeatures & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDeclarativeGeoServiceProviderRequirements::routingRequirementsChanged(const QDeclarativeGeoServiceProvider::RoutingFeatures & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QDeclarativeGeoServiceProviderRequirements::geocodingRequirementsChanged(const QDeclarativeGeoServiceProvider::GeocodingFeatures & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QDeclarativeGeoServiceProviderRequirements::placesRequirementsChanged(const QDeclarativeGeoServiceProvider::PlacesFeatures & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QDeclarativeGeoServiceProviderRequirements::navigationRequirementsChanged(const QDeclarativeGeoServiceProvider::NavigationFeatures & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QDeclarativeGeoServiceProviderRequirements::requirementsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
