/****************************************************************************
** Meta object code from reading C++ file 'unsupportedreplies_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/places/unsupportedreplies_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'unsupportedreplies_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29QPlaceDetailsReplyUnsupportedE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceDetailsReplyUnsupported::qt_create_metaobjectdata<qt_meta_tag_ZN29QPlaceDetailsReplyUnsupportedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceDetailsReplyUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceDetailsReplyUnsupported, qt_meta_tag_ZN29QPlaceDetailsReplyUnsupportedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceDetailsReplyUnsupported::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlaceDetailsReply::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QPlaceDetailsReplyUnsupportedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QPlaceDetailsReplyUnsupportedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QPlaceDetailsReplyUnsupportedE_t>.metaTypes,
    nullptr
} };

void QPlaceDetailsReplyUnsupported::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceDetailsReplyUnsupported *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QPlaceDetailsReplyUnsupported::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceDetailsReplyUnsupported::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QPlaceDetailsReplyUnsupportedE_t>.strings))
        return static_cast<void*>(this);
    return QPlaceDetailsReply::qt_metacast(_clname);
}

int QPlaceDetailsReplyUnsupported::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlaceDetailsReply::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN29QPlaceContentReplyUnsupportedE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceContentReplyUnsupported::qt_create_metaobjectdata<qt_meta_tag_ZN29QPlaceContentReplyUnsupportedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceContentReplyUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceContentReplyUnsupported, qt_meta_tag_ZN29QPlaceContentReplyUnsupportedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceContentReplyUnsupported::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlaceContentReply::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QPlaceContentReplyUnsupportedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QPlaceContentReplyUnsupportedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QPlaceContentReplyUnsupportedE_t>.metaTypes,
    nullptr
} };

void QPlaceContentReplyUnsupported::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceContentReplyUnsupported *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QPlaceContentReplyUnsupported::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceContentReplyUnsupported::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QPlaceContentReplyUnsupportedE_t>.strings))
        return static_cast<void*>(this);
    return QPlaceContentReply::qt_metacast(_clname);
}

int QPlaceContentReplyUnsupported::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlaceContentReply::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN28QPlaceSearchReplyUnsupportedE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceSearchReplyUnsupported::qt_create_metaobjectdata<qt_meta_tag_ZN28QPlaceSearchReplyUnsupportedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceSearchReplyUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceSearchReplyUnsupported, qt_meta_tag_ZN28QPlaceSearchReplyUnsupportedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceSearchReplyUnsupported::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlaceSearchReply::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QPlaceSearchReplyUnsupportedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QPlaceSearchReplyUnsupportedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QPlaceSearchReplyUnsupportedE_t>.metaTypes,
    nullptr
} };

void QPlaceSearchReplyUnsupported::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceSearchReplyUnsupported *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QPlaceSearchReplyUnsupported::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceSearchReplyUnsupported::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QPlaceSearchReplyUnsupportedE_t>.strings))
        return static_cast<void*>(this);
    return QPlaceSearchReply::qt_metacast(_clname);
}

int QPlaceSearchReplyUnsupported::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlaceSearchReply::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN38QPlaceSearchSuggestionReplyUnsupportedE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceSearchSuggestionReplyUnsupported::qt_create_metaobjectdata<qt_meta_tag_ZN38QPlaceSearchSuggestionReplyUnsupportedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceSearchSuggestionReplyUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceSearchSuggestionReplyUnsupported, qt_meta_tag_ZN38QPlaceSearchSuggestionReplyUnsupportedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceSearchSuggestionReplyUnsupported::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlaceSearchSuggestionReply::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38QPlaceSearchSuggestionReplyUnsupportedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38QPlaceSearchSuggestionReplyUnsupportedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN38QPlaceSearchSuggestionReplyUnsupportedE_t>.metaTypes,
    nullptr
} };

void QPlaceSearchSuggestionReplyUnsupported::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceSearchSuggestionReplyUnsupported *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QPlaceSearchSuggestionReplyUnsupported::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceSearchSuggestionReplyUnsupported::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38QPlaceSearchSuggestionReplyUnsupportedE_t>.strings))
        return static_cast<void*>(this);
    return QPlaceSearchSuggestionReply::qt_metacast(_clname);
}

int QPlaceSearchSuggestionReplyUnsupported::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlaceSearchSuggestionReply::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN24QPlaceIdReplyUnsupportedE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceIdReplyUnsupported::qt_create_metaobjectdata<qt_meta_tag_ZN24QPlaceIdReplyUnsupportedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceIdReplyUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceIdReplyUnsupported, qt_meta_tag_ZN24QPlaceIdReplyUnsupportedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceIdReplyUnsupported::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlaceIdReply::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QPlaceIdReplyUnsupportedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QPlaceIdReplyUnsupportedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QPlaceIdReplyUnsupportedE_t>.metaTypes,
    nullptr
} };

void QPlaceIdReplyUnsupported::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceIdReplyUnsupported *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QPlaceIdReplyUnsupported::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceIdReplyUnsupported::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QPlaceIdReplyUnsupportedE_t>.strings))
        return static_cast<void*>(this);
    return QPlaceIdReply::qt_metacast(_clname);
}

int QPlaceIdReplyUnsupported::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlaceIdReply::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN22QPlaceReplyUnsupportedE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceReplyUnsupported::qt_create_metaobjectdata<qt_meta_tag_ZN22QPlaceReplyUnsupportedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceReplyUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceReplyUnsupported, qt_meta_tag_ZN22QPlaceReplyUnsupportedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceReplyUnsupported::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlaceReply::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QPlaceReplyUnsupportedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QPlaceReplyUnsupportedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22QPlaceReplyUnsupportedE_t>.metaTypes,
    nullptr
} };

void QPlaceReplyUnsupported::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceReplyUnsupported *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QPlaceReplyUnsupported::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceReplyUnsupported::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QPlaceReplyUnsupportedE_t>.strings))
        return static_cast<void*>(this);
    return QPlaceReply::qt_metacast(_clname);
}

int QPlaceReplyUnsupported::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlaceReply::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN27QPlaceMatchReplyUnsupportedE_t {};
} // unnamed namespace

template <> constexpr inline auto QPlaceMatchReplyUnsupported::qt_create_metaobjectdata<qt_meta_tag_ZN27QPlaceMatchReplyUnsupportedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPlaceMatchReplyUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPlaceMatchReplyUnsupported, qt_meta_tag_ZN27QPlaceMatchReplyUnsupportedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPlaceMatchReplyUnsupported::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlaceMatchReply::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QPlaceMatchReplyUnsupportedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QPlaceMatchReplyUnsupportedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QPlaceMatchReplyUnsupportedE_t>.metaTypes,
    nullptr
} };

void QPlaceMatchReplyUnsupported::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPlaceMatchReplyUnsupported *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QPlaceMatchReplyUnsupported::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPlaceMatchReplyUnsupported::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QPlaceMatchReplyUnsupportedE_t>.strings))
        return static_cast<void*>(this);
    return QPlaceMatchReply::qt_metacast(_clname);
}

int QPlaceMatchReplyUnsupported::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlaceMatchReply::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
