/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeomapitemview_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/quickmapitems/qdeclarativegeomapitemview_p.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeomapitemview_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26QDeclarativeGeoMapItemViewE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoMapItemView::qt_create_metaobjectdata<qt_meta_tag_ZN26QDeclarativeGeoMapItemViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoMapItemView",
        "QML.Element",
        "MapItemView",
        "QML.AddedInVersion",
        "1280",
        "modelChanged",
        "",
        "delegateChanged",
        "autoFitViewportChanged",
        "incubateDelegatesChanged",
        "delegateModelAccessChanged",
        "destroyingItem",
        "object",
        "initItem",
        "index",
        "createdItem",
        "modelUpdated",
        "QQmlChangeSet",
        "changeSet",
        "reset",
        "exitTransitionFinished",
        "model",
        "QVariant",
        "delegate",
        "QQmlComponent*",
        "autoFitViewport",
        "add",
        "QQuickTransition*",
        "remove",
        "mapItems",
        "QList<QQuickItem*>",
        "incubateDelegates",
        "delegateModelAccess",
        "QQmlDelegateModel::DelegateModelAccess"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modelChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'delegateChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoFitViewportChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'incubateDelegatesChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'delegateModelAccessChanged'
        QtMocHelpers::RevisionedSignalData<void()>(10, 6, QMC::AccessPublic, 0x60a, QMetaType::Void),
        // Slot 'destroyingItem'
        QtMocHelpers::SlotData<void(QObject *)>(11, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 12 },
        }}),
        // Slot 'initItem'
        QtMocHelpers::SlotData<void(int, QObject *)>(13, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 }, { QMetaType::QObjectStar, 12 },
        }}),
        // Slot 'createdItem'
        QtMocHelpers::SlotData<void(int, QObject *)>(15, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 }, { QMetaType::QObjectStar, 12 },
        }}),
        // Slot 'modelUpdated'
        QtMocHelpers::SlotData<void(const QQmlChangeSet &, bool)>(16, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'exitTransitionFinished'
        QtMocHelpers::SlotData<void()>(20, 6, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'model'
        QtMocHelpers::PropertyData<QVariant>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'delegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'autoFitViewport'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'add'
        QtMocHelpers::PropertyData<QQuickTransition*>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, -1, 0x50c),
        // property 'remove'
        QtMocHelpers::PropertyData<QQuickTransition*>(28, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, -1, 0x50c),
        // property 'mapItems'
        QtMocHelpers::PropertyData<QList<QQuickItem*>>(29, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, -1, 0x50c),
        // property 'incubateDelegates'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3, 0x50c),
        // property 'delegateModelAccess'
        QtMocHelpers::PropertyData<QQmlDelegateModel::DelegateModelAccess>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4, 0x60a),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoMapItemView, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN26QDeclarativeGeoMapItemViewE[] = {
    QMetaObject::SuperData::link<QQmlDelegateModel::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QDeclarativeGeoMapItemView::staticMetaObject = { {
    QMetaObject::SuperData::link<QDeclarativeGeoMapItemGroup::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemViewE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN26QDeclarativeGeoMapItemViewE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemViewE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoMapItemView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoMapItemView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modelChanged(); break;
        case 1: _t->delegateChanged(); break;
        case 2: _t->autoFitViewportChanged(); break;
        case 3: _t->incubateDelegatesChanged(); break;
        case 4: _t->delegateModelAccessChanged(); break;
        case 5: _t->destroyingItem((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 6: _t->initItem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 7: _t->createdItem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 8: _t->modelUpdated((*reinterpret_cast<std::add_pointer_t<QQmlChangeSet>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->exitTransitionFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemView::*)()>(_a, &QDeclarativeGeoMapItemView::modelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemView::*)()>(_a, &QDeclarativeGeoMapItemView::delegateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemView::*)()>(_a, &QDeclarativeGeoMapItemView::autoFitViewportChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemView::*)()>(_a, &QDeclarativeGeoMapItemView::incubateDelegatesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemView::*)()>(_a, &QDeclarativeGeoMapItemView::delegateModelAccessChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QQuickItem*> >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickTransition* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariant*>(_v) = _t->model(); break;
        case 1: *reinterpret_cast<QQmlComponent**>(_v) = _t->delegate(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->autoFitViewport(); break;
        case 3: *reinterpret_cast<QQuickTransition**>(_v) = _t->m_enter; break;
        case 4: *reinterpret_cast<QQuickTransition**>(_v) = _t->m_exit; break;
        case 5: *reinterpret_cast<QList<QQuickItem*>*>(_v) = _t->mapItems(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->incubateDelegates(); break;
        case 7: *reinterpret_cast<QQmlDelegateModel::DelegateModelAccess*>(_v) = _t->delegateModelAccess(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast<QVariant*>(_v)); break;
        case 1: _t->setDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        case 2: _t->setAutoFitViewport(*reinterpret_cast<bool*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->m_enter, *reinterpret_cast<QQuickTransition**>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->m_exit, *reinterpret_cast<QQuickTransition**>(_v)); break;
        case 6: _t->setIncubateDelegates(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setDelegateModelAccess(*reinterpret_cast<QQmlDelegateModel::DelegateModelAccess*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoMapItemView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoMapItemView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemViewE_t>.strings))
        return static_cast<void*>(this);
    return QDeclarativeGeoMapItemGroup::qt_metacast(_clname);
}

int QDeclarativeGeoMapItemView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDeclarativeGeoMapItemGroup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoMapItemView::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeGeoMapItemView::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoMapItemView::autoFitViewportChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeoMapItemView::incubateDelegatesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeGeoMapItemView::delegateModelAccessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
