/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativeplace_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativeplaces/qdeclarativeplace_p.h"
#include <QtLocation/private/qdeclarativeplace_p.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativeplace_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QDeclarativePlaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativePlace::qt_create_metaobjectdata<qt_meta_tag_ZN17QDeclarativePlaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativePlace",
        "QML.Element",
        "Place",
        "QML.AddedInVersion",
        "1280",
        "pluginChanged",
        "",
        "categoriesChanged",
        "locationChanged",
        "ratingsChanged",
        "supplierChanged",
        "iconChanged",
        "nameChanged",
        "placeIdChanged",
        "attributionChanged",
        "detailsFetchedChanged",
        "reviewModelChanged",
        "imageModelChanged",
        "editorialModelChanged",
        "primaryPhoneChanged",
        "primaryFaxChanged",
        "primaryEmailChanged",
        "primaryWebsiteChanged",
        "extendedAttributesChanged",
        "contactDetailsChanged",
        "statusChanged",
        "visibilityChanged",
        "favoriteChanged",
        "finished",
        "contactsModified",
        "QVariant",
        "pluginReady",
        "cleanupDeletedCategories",
        "getDetails",
        "save",
        "remove",
        "errorString",
        "copyFrom",
        "QDeclarativePlace*",
        "original",
        "initializeFavorite",
        "QDeclarativeGeoServiceProvider*",
        "plugin",
        "place",
        "QPlace",
        "categories",
        "QQmlListProperty<QDeclarativeCategory>",
        "location",
        "QDeclarativeGeoLocation*",
        "ratings",
        "QPlaceRatings",
        "supplier",
        "QPlaceSupplier",
        "icon",
        "QPlaceIcon",
        "name",
        "placeId",
        "attribution",
        "reviewModel",
        "QDeclarativePlaceReviewModel*",
        "imageModel",
        "QDeclarativePlaceImageModel*",
        "editorialModel",
        "QDeclarativePlaceEditorialModel*",
        "extendedAttributes",
        "contactDetails",
        "QDeclarativeContactDetails*",
        "detailsFetched",
        "status",
        "Status",
        "primaryPhone",
        "primaryFax",
        "primaryEmail",
        "primaryWebsite",
        "QUrl",
        "visibility",
        "Visibility",
        "favorite",
        "Ready",
        "Saving",
        "Fetching",
        "Removing",
        "Error",
        "UnspecifiedVisibility",
        "DeviceVisibility",
        "PrivateVisibility",
        "PublicVisibility"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pluginChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'categoriesChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'locationChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ratingsChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'supplierChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'placeIdChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'attributionChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'detailsFetchedChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reviewModelChanged'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imageModelChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'editorialModelChanged'
        QtMocHelpers::SignalData<void()>(18, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryPhoneChanged'
        QtMocHelpers::SignalData<void()>(19, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryFaxChanged'
        QtMocHelpers::SignalData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryEmailChanged'
        QtMocHelpers::SignalData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryWebsiteChanged'
        QtMocHelpers::SignalData<void()>(22, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'extendedAttributesChanged'
        QtMocHelpers::SignalData<void()>(23, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contactDetailsChanged'
        QtMocHelpers::SignalData<void()>(24, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(25, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibilityChanged'
        QtMocHelpers::SignalData<void()>(26, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'favoriteChanged'
        QtMocHelpers::SignalData<void()>(27, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'finished'
        QtMocHelpers::SlotData<void()>(28, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'contactsModified'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(29, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 30, 6 },
        }}),
        // Slot 'pluginReady'
        QtMocHelpers::SlotData<void()>(31, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupDeletedCategories'
        QtMocHelpers::SlotData<void()>(32, 6, QMC::AccessPrivate, QMetaType::Void),
        // Method 'getDetails'
        QtMocHelpers::MethodData<void()>(33, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'save'
        QtMocHelpers::MethodData<void()>(34, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'remove'
        QtMocHelpers::MethodData<void()>(35, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'errorString'
        QtMocHelpers::MethodData<QString() const>(36, 6, QMC::AccessPublic, QMetaType::QString),
        // Method 'copyFrom'
        QtMocHelpers::MethodData<void(QDeclarativePlace *)>(37, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Method 'initializeFavorite'
        QtMocHelpers::MethodData<void(QDeclarativeGeoServiceProvider *)>(40, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 42 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'place'
        QtMocHelpers::PropertyData<QPlace>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'plugin'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider*>(42, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'categories'
        QtMocHelpers::PropertyData<QQmlListProperty<QDeclarativeCategory>>(45, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'location'
        QtMocHelpers::PropertyData<QDeclarativeGeoLocation*>(47, 0x80000000 | 48, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'ratings'
        QtMocHelpers::PropertyData<QPlaceRatings>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'supplier'
        QtMocHelpers::PropertyData<QPlaceSupplier>(51, 0x80000000 | 52, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'icon'
        QtMocHelpers::PropertyData<QPlaceIcon>(53, 0x80000000 | 54, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'placeId'
        QtMocHelpers::PropertyData<QString>(56, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'attribution'
        QtMocHelpers::PropertyData<QString>(57, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'reviewModel'
        QtMocHelpers::PropertyData<QDeclarativePlaceReviewModel*>(58, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 10),
        // property 'imageModel'
        QtMocHelpers::PropertyData<QDeclarativePlaceImageModel*>(60, 0x80000000 | 61, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 11),
        // property 'editorialModel'
        QtMocHelpers::PropertyData<QDeclarativePlaceEditorialModel*>(62, 0x80000000 | 63, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 12),
        // property 'extendedAttributes'
        QtMocHelpers::PropertyData<QObject*>(64, QMetaType::QObjectStar, QMC::DefaultPropertyFlags, 17),
        // property 'contactDetails'
        QtMocHelpers::PropertyData<QDeclarativeContactDetails*>(65, 0x80000000 | 66, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 18),
        // property 'detailsFetched'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags, 9),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(68, 0x80000000 | 69, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 19),
        // property 'primaryPhone'
        QtMocHelpers::PropertyData<QString>(70, QMetaType::QString, QMC::DefaultPropertyFlags, 13),
        // property 'primaryFax'
        QtMocHelpers::PropertyData<QString>(71, QMetaType::QString, QMC::DefaultPropertyFlags, 14),
        // property 'primaryEmail'
        QtMocHelpers::PropertyData<QString>(72, QMetaType::QString, QMC::DefaultPropertyFlags, 15),
        // property 'primaryWebsite'
        QtMocHelpers::PropertyData<QUrl>(73, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 16),
        // property 'visibility'
        QtMocHelpers::PropertyData<enum Visibility>(75, 0x80000000 | 76, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 20),
        // property 'favorite'
        QtMocHelpers::PropertyData<QDeclarativePlace*>(77, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 21),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(69, 69, QMC::EnumFlags{}).add({
            {   78, Status::Ready },
            {   79, Status::Saving },
            {   80, Status::Fetching },
            {   81, Status::Removing },
            {   82, Status::Error },
        }),
        // enum 'Visibility'
        QtMocHelpers::EnumData<enum Visibility>(76, 76, QMC::EnumFlags{}).add({
            {   83, Visibility::UnspecifiedVisibility },
            {   84, Visibility::DeviceVisibility },
            {   85, Visibility::PrivateVisibility },
            {   86, Visibility::PublicVisibility },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativePlace, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativePlace::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QDeclarativePlaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QDeclarativePlaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QDeclarativePlaceE_t>.metaTypes,
    nullptr
} };

void QDeclarativePlace::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativePlace *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pluginChanged(); break;
        case 1: _t->categoriesChanged(); break;
        case 2: _t->locationChanged(); break;
        case 3: _t->ratingsChanged(); break;
        case 4: _t->supplierChanged(); break;
        case 5: _t->iconChanged(); break;
        case 6: _t->nameChanged(); break;
        case 7: _t->placeIdChanged(); break;
        case 8: _t->attributionChanged(); break;
        case 9: _t->detailsFetchedChanged(); break;
        case 10: _t->reviewModelChanged(); break;
        case 11: _t->imageModelChanged(); break;
        case 12: _t->editorialModelChanged(); break;
        case 13: _t->primaryPhoneChanged(); break;
        case 14: _t->primaryFaxChanged(); break;
        case 15: _t->primaryEmailChanged(); break;
        case 16: _t->primaryWebsiteChanged(); break;
        case 17: _t->extendedAttributesChanged(); break;
        case 18: _t->contactDetailsChanged(); break;
        case 19: _t->statusChanged(); break;
        case 20: _t->visibilityChanged(); break;
        case 21: _t->favoriteChanged(); break;
        case 22: _t->finished(); break;
        case 23: _t->contactsModified((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 24: _t->pluginReady(); break;
        case 25: _t->cleanupDeletedCategories(); break;
        case 26: _t->getDetails(); break;
        case 27: _t->save(); break;
        case 28: _t->remove(); break;
        case 29: { QString _r = _t->errorString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->copyFrom((*reinterpret_cast<std::add_pointer_t<QDeclarativePlace*>>(_a[1]))); break;
        case 31: _t->initializeFavorite((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoServiceProvider*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDeclarativePlace* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDeclarativeGeoServiceProvider* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::pluginChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::categoriesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::locationChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::ratingsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::supplierChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::iconChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::nameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::placeIdChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::attributionChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::detailsFetchedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::reviewModelChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::imageModelChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::editorialModelChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::primaryPhoneChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::primaryFaxChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::primaryEmailChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::primaryWebsiteChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::extendedAttributesChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::contactDetailsChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::statusChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::visibilityChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativePlace::*)()>(_a, &QDeclarativePlace::favoriteChanged, 21))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoLocation* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoServiceProvider* >(); break;
        case 22:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativePlace* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPlace >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPlaceIcon >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPlaceRatings >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPlaceSupplier >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativeCategory> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QPlace*>(_v) = _t->place(); break;
        case 1: *reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v) = _t->plugin(); break;
        case 2: *reinterpret_cast<QQmlListProperty<QDeclarativeCategory>*>(_v) = _t->categories(); break;
        case 3: *reinterpret_cast<QDeclarativeGeoLocation**>(_v) = _t->location(); break;
        case 4: *reinterpret_cast<QPlaceRatings*>(_v) = _t->ratings(); break;
        case 5: *reinterpret_cast<QPlaceSupplier*>(_v) = _t->supplier(); break;
        case 6: *reinterpret_cast<QPlaceIcon*>(_v) = _t->icon(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->placeId(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->attribution(); break;
        case 10: *reinterpret_cast<QDeclarativePlaceReviewModel**>(_v) = _t->reviewModel(); break;
        case 11: *reinterpret_cast<QDeclarativePlaceImageModel**>(_v) = _t->imageModel(); break;
        case 12: *reinterpret_cast<QDeclarativePlaceEditorialModel**>(_v) = _t->editorialModel(); break;
        case 13: *reinterpret_cast<QObject**>(_v) = _t->extendedAttributes(); break;
        case 14: *reinterpret_cast<QDeclarativeContactDetails**>(_v) = _t->contactDetails(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->detailsFetched(); break;
        case 16: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->primaryPhone(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->primaryFax(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->primaryEmail(); break;
        case 20: *reinterpret_cast<QUrl*>(_v) = _t->primaryWebsite(); break;
        case 21: *reinterpret_cast<enum Visibility*>(_v) = _t->visibility(); break;
        case 22: *reinterpret_cast<QDeclarativePlace**>(_v) = _t->favorite(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlace(*reinterpret_cast<QPlace*>(_v)); break;
        case 1: _t->setPlugin(*reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v)); break;
        case 3: _t->setLocation(*reinterpret_cast<QDeclarativeGeoLocation**>(_v)); break;
        case 4: _t->setRatings(*reinterpret_cast<QPlaceRatings*>(_v)); break;
        case 5: _t->setSupplier(*reinterpret_cast<QPlaceSupplier*>(_v)); break;
        case 6: _t->setIcon(*reinterpret_cast<QPlaceIcon*>(_v)); break;
        case 7: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setPlaceId(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setAttribution(*reinterpret_cast<QString*>(_v)); break;
        case 21: _t->setVisibility(*reinterpret_cast<enum Visibility*>(_v)); break;
        case 22: _t->setFavorite(*reinterpret_cast<QDeclarativePlace**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativePlace::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativePlace::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QDeclarativePlaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativePlace::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativePlace::pluginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativePlace::categoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativePlace::locationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativePlace::ratingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativePlace::supplierChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativePlace::iconChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativePlace::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativePlace::placeIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativePlace::attributionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QDeclarativePlace::detailsFetchedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QDeclarativePlace::reviewModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QDeclarativePlace::imageModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QDeclarativePlace::editorialModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QDeclarativePlace::primaryPhoneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QDeclarativePlace::primaryFaxChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QDeclarativePlace::primaryEmailChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QDeclarativePlace::primaryWebsiteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QDeclarativePlace::extendedAttributesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QDeclarativePlace::contactDetailsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QDeclarativePlace::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QDeclarativePlace::visibilityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QDeclarativePlace::favoriteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}
QT_WARNING_POP
