/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativesearchresultmodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativeplaces/qdeclarativesearchresultmodel_p.h"
#include <QtLocation/private/qdeclarativeplace_p.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativesearchresultmodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29QDeclarativeSearchResultModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeSearchResultModel::qt_create_metaobjectdata<qt_meta_tag_ZN29QDeclarativeSearchResultModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeSearchResultModel",
        "QML.Element",
        "PlaceSearchModel",
        "QML.AddedInVersion",
        "1280",
        "searchTermChanged",
        "",
        "categoriesChanged",
        "recommendationIdChanged",
        "relevanceHintChanged",
        "visibilityScopeChanged",
        "rowCountChanged",
        "favoritesPluginChanged",
        "favoritesMatchParametersChanged",
        "dataChanged",
        "incrementalChanged",
        "queryFinished",
        "onContentUpdated",
        "updateLayout",
        "QList<QPlace>",
        "favoritePlaces",
        "placeUpdated",
        "placeId",
        "placeRemoved",
        "data",
        "QVariant",
        "index",
        "roleName",
        "updateWith",
        "proposedSearchIndex",
        "searchTerm",
        "categories",
        "QQmlListProperty<QDeclarativeCategory>",
        "recommendationId",
        "relevanceHint",
        "RelevanceHint",
        "visibilityScope",
        "QDeclarativePlace::Visibility",
        "count",
        "favoritesPlugin",
        "QDeclarativeGeoServiceProvider*",
        "favoritesMatchParameters",
        "QVariantMap",
        "incremental",
        "SearchResultType",
        "UnknownSearchResult",
        "PlaceResult",
        "ProposedSearchResult",
        "UnspecifiedHint",
        "DistanceHint",
        "LexicalPlaceNameHint"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'searchTermChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'categoriesChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'recommendationIdChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'relevanceHintChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibilityScopeChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rowCountChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'favoritesPluginChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'favoritesMatchParametersChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'incrementalChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'queryFinished'
        QtMocHelpers::SlotData<void()>(16, 6, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onContentUpdated'
        QtMocHelpers::SlotData<void()>(17, 6, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateLayout'
        QtMocHelpers::SlotData<void(const QList<QPlace> &)>(18, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'updateLayout'
        QtMocHelpers::SlotData<void()>(18, 6, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'placeUpdated'
        QtMocHelpers::SlotData<void(const QString &)>(21, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'placeRemoved'
        QtMocHelpers::SlotData<void(const QString &)>(23, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'data'
        QtMocHelpers::MethodData<QVariant(int, const QString &) const>(24, 6, QMC::AccessPublic, 0x80000000 | 25, {{
            { QMetaType::Int, 26 }, { QMetaType::QString, 27 },
        }}),
        // Method 'updateWith'
        QtMocHelpers::MethodData<void(int)>(28, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'searchTerm'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'categories'
        QtMocHelpers::PropertyData<QQmlListProperty<QDeclarativeCategory>>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'recommendationId'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'relevanceHint'
        QtMocHelpers::PropertyData<enum RelevanceHint>(34, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'visibilityScope'
        QtMocHelpers::PropertyData<QDeclarativePlace::Visibility>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'count'
        QtMocHelpers::PropertyData<int>(38, QMetaType::Int, QMC::DefaultPropertyFlags, 5),
        // property 'favoritesPlugin'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider*>(39, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'favoritesMatchParameters'
        QtMocHelpers::PropertyData<QVariantMap>(41, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'incremental'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 9, 0x50c),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SearchResultType'
        QtMocHelpers::EnumData<enum SearchResultType>(44, 44, QMC::EnumFlags{}).add({
            {   45, SearchResultType::UnknownSearchResult },
            {   46, SearchResultType::PlaceResult },
            {   47, SearchResultType::ProposedSearchResult },
        }),
        // enum 'RelevanceHint'
        QtMocHelpers::EnumData<enum RelevanceHint>(35, 35, QMC::EnumFlags{}).add({
            {   48, RelevanceHint::UnspecifiedHint },
            {   49, RelevanceHint::DistanceHint },
            {   50, RelevanceHint::LexicalPlaceNameHint },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeSearchResultModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN29QDeclarativeSearchResultModelE[] = {
    QMetaObject::SuperData::link<QDeclarativePlace::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QDeclarativeSearchResultModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QDeclarativeSearchModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativeSearchResultModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativeSearchResultModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN29QDeclarativeSearchResultModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QDeclarativeSearchResultModelE_t>.metaTypes,
    nullptr
} };

void QDeclarativeSearchResultModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeSearchResultModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->searchTermChanged(); break;
        case 1: _t->categoriesChanged(); break;
        case 2: _t->recommendationIdChanged(); break;
        case 3: _t->relevanceHintChanged(); break;
        case 4: _t->visibilityScopeChanged(); break;
        case 5: _t->rowCountChanged(); break;
        case 6: _t->favoritesPluginChanged(); break;
        case 7: _t->favoritesMatchParametersChanged(); break;
        case 8: _t->dataChanged(); break;
        case 9: _t->incrementalChanged(); break;
        case 10: _t->queryFinished(); break;
        case 11: _t->onContentUpdated(); break;
        case 12: _t->updateLayout((*reinterpret_cast<std::add_pointer_t<QList<QPlace>>>(_a[1]))); break;
        case 13: _t->updateLayout(); break;
        case 14: _t->placeUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->placeRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: { QVariant _r = _t->data((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->updateWith((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPlace> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::searchTermChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::categoriesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::recommendationIdChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::relevanceHintChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::visibilityScopeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::rowCountChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::favoritesPluginChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::favoritesMatchParametersChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::dataChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeSearchResultModel::*)()>(_a, &QDeclarativeSearchResultModel::incrementalChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoServiceProvider* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativeCategory> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->searchTerm(); break;
        case 1: *reinterpret_cast<QQmlListProperty<QDeclarativeCategory>*>(_v) = _t->categories(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->recommendationId(); break;
        case 3: *reinterpret_cast<enum RelevanceHint*>(_v) = _t->relevanceHint(); break;
        case 4: *reinterpret_cast<QDeclarativePlace::Visibility*>(_v) = _t->visibilityScope(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->rowCount(); break;
        case 6: *reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v) = _t->favoritesPlugin(); break;
        case 7: *reinterpret_cast<QVariantMap*>(_v) = _t->favoritesMatchParameters(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->m_incremental; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSearchTerm(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setRecommendationId(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setRelevanceHint(*reinterpret_cast<enum RelevanceHint*>(_v)); break;
        case 4: _t->setVisibilityScope(*reinterpret_cast<QDeclarativePlace::Visibility*>(_v)); break;
        case 6: _t->setFavoritesPlugin(*reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v)); break;
        case 7: _t->setFavoritesMatchParameters(*reinterpret_cast<QVariantMap*>(_v)); break;
        case 8:
            if (QtMocHelpers::setProperty(_t->m_incremental, *reinterpret_cast<bool*>(_v)))
                Q_EMIT _t->incrementalChanged();
            break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeSearchResultModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeSearchResultModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativeSearchResultModelE_t>.strings))
        return static_cast<void*>(this);
    return QDeclarativeSearchModelBase::qt_metacast(_clname);
}

int QDeclarativeSearchResultModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDeclarativeSearchModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeSearchResultModel::searchTermChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeSearchResultModel::categoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeSearchResultModel::recommendationIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeSearchResultModel::relevanceHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeSearchResultModel::visibilityScopeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeSearchResultModel::rowCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeSearchResultModel::favoritesPluginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativeSearchResultModel::favoritesMatchParametersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativeSearchResultModel::dataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QDeclarativeSearchResultModel::incrementalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
